<?php
/**
 * File containing the abstract class GUI.
 * 
 * @category Class
 * @package PHPSupportTickets
 * @author Ian Warner, <iwarner@triangle-solutions.com> 
 * @author Nicolas Connault, <nicou@sweetpeadesigns.com.au> 
 * @copyright (C) 2001 Triangle Solutions Ltd
 * @version CVS: $Id: abstract.GUI.php,v 1.21 2005/09/02 03:12:44 nicconn Exp $
 * @since File available since Release 1.1.1.1
 * \\||
 */

/**
* user defined includes.
* @include
*/
require_once('classes/static/static.FormHandler.php');
define('PHPST_MAX_RECORDS', 10);

/**
 * GUI is an abstraction of a User Interface.
 * 
 * @abstract
 * @access public
 * @package PHPSupportTicket
 */
abstract class GUI {
    /// --- FIELDS ---
    
    /**
    * The User object using this GUI.
    * @access public
    * @var object $user
    */
    public $user;
    
    /**
    * The absolute URL for this GUI.
    * @access private
    * @var string $page
    */
    private $absoluteURL;
    
    /**
    * The current page for this GUI.
    * @access private
    * @var string $page
    */
    private $page;
    
    /**
    * A message about the last action performed, usually success or failure.
    * 
    * @access private
    * @var string $msg
    */
    private $msg;
    
    /**
    * The XHTML code designed to be output to the browser.
    * 
    * @access private
    * @var string $html
    */
    private $html;
        
    // --- METHODS ---
    
    /**
    * Constructor.
    * 
    * @access public
    * @param string $absoluteURL
    * @param array $request
    */
    public function __construct($absoluteURL, $request) {
        $this->setAbsoluteURL($absoluteURL);
        
        // Do not retrieve user if logout or new page
        if (!isset($request['logout']) && count($request) > 1) {
            $this->user = User::getFromDB($request);
            $this->user->setLast_login(time());
            $this->user->updateDB();
        }
    }
    
    /**
    * Used to output the HTML code to the browser.
    * 
    * @access public
    * @return void
    */
    public function output() {
        print $this->getHTML();
    }
    
    /**
     * GUI::addHTML()
     * 
     * @access public
     * @param string $string
     * @return 
     **/
    public function addHTML($string) {
        $this->html .= $string;
    }
    
    /**
     * GUI::getHTML()
     * 
     * @access public
     * @return string
     **/
    public function getHTML() {
        return $this->html;
    }
    
    /**
     * Returns this GUI's absolute URL
     * 
     * @access public
     * @param $absoluteURL
     * @return string
     **/
    public function getAbsoluteURL() {
        return $this->absoluteURL;
    }
    
    /**
     * Sets this GUI's absolute URL
     * 
     * @access public
     * @param $absoluteURL
     * @return void
     **/
    public function setAbsoluteURL($absoluteURL) {
        $this->absoluteURL = $absoluteURL;
    }
    
    /**
     * Adds the header to this GUI's HTML.
     * 
     * @param string $title
     * @param string $base
     * @return void
     **/
    public function buildHeader($title = '') {
        $base = $this->getAbsoluteURL();
        $title = PHPST_PHPST . ' :: ' . $title;
        $string = '
            <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
            
            <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
            <head>
            <title>' . $title . '</title>
            <base href="' . $base . '" />
            <meta http-equiv="content-type" content="text/xml; charset=iso-8859-1" />
            
            <meta name="description" content="" />
            <meta name="keywords"    content="" />
            <meta name="author"      content="Ian Warner" />
            <meta name="copyright"   content="Triangle Solutions Ltd" />
            <meta name="rating"      content="General" />
            <meta name="robots"      content="All" />
            
            <link rel="stylesheet"    href="include/styles.css" type="text/css" />
            
            <link rel="shortcut icon" href="favicon.ico" />
            <script language="javascript" type="text/javascript">
        	<!-- Hide script from old browsers
        
        	function popwindow(popsrc1,stuff1)
        		{
        		videoWindow = window.open(popsrc1, \'POPUP\', stuff1)
        		}
        
        	function placeFocus()
        		{
        		if (document.forms.length > 0)
        			{
        			var field = document.forms[0];
        			for (i = 0; i < field.length; i++)
        				{
        			if ((field.elements[i].type == "text") || (field.elements[i].type == "textarea") || (field.elements[i].type.toString().charAt(0) == "s"))
        					{
        					document.forms[0].elements[i].focus();
        					break;
        					}
        				}
        			}
        		}
        
        	// End hiding script from old browsers -->
        	</script>
            </head>
            <body>';
        $this->addHTML($string);
    }
    
   /**
    * Builds the viewticket page.
    * 
    * @access public
    * @return string
    * @param array $request
    * @param array $tickets
    */
    public function buildViewticket($request, $ticket) {
        $url = $this->getAbsoluteUrl() . 'index.php';
        $field = '';
        if (is_array($this->getMsg())) {
            $msg = $this->getMsg();
            $error = $msg[0]['field'];
            $this->setMsg($msg[0]['message']);
        } 
        $admin_type = $this->user->getAdmin_status();
        
        // Check if there is a file attached to this ticket
        $ticket_attachment_path = PHPST_UPLOAD_PATH . $ticket['user']['username']
                . '/ticket_' . $ticket['ticket']['ID'];
        
        if (is_dir($ticket_attachment_path)) {
            // Prepare path to file and filename for linking
            $contents = scandir($ticket_attachment_path);
            $ticket_attachment = $contents[2];
            $link_to_file = PHPST_UPLOAD_RELATIVE_PATH . $ticket['user']['username'] 
                    . '/ticket_' . $ticket['ticket']['ID'] . '/' . $ticket_attachment;
            $attachment = PHPST_ATTACHMENT;
            $download_image = '<img border="0" src="images/download.gif" />';
        }
        
        // If there are no answers for this ticket, just create an empty array
        if (!isset($ticket['answers'])) {
            $ticket['answers'] = array();
        }
        // print_r($tickets);
        $string = $this->buildNavigation($request);
        $string .= '
            <form enctype="multipart/form-data" action="' . $url . '" method="post">
                <input type="hidden" name="formdata" value="addanswer" />
                <input type="hidden" name="page" value="viewticket" />
                <input type="hidden" name="field" value="tickets.ID" />
                <input type="hidden" name="search" value="' . $ticket['ticket']['ID'] . '" />
                <input type="hidden" name="user_ID" value="' . $this->user->getID() . '" />
                <input type="hidden" name="ticket_ID" value="' . $ticket['ticket']['ID'] . '" />
                <input type="hidden" name="subject" value="' . $ticket['ticket']['subject'] . '" />
                <input type="hidden" name="timestamp" value="' . time() . '" />
                <input type="hidden" name="rating" value="1" />
            <table width="75%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
              <tr>
                <td class="boxborder" width="50%" valign="top" style="padding-top:5px">
                
                  <table width="97%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                    <tr>
                      <td bgcolor="#AABBDD" class="boxborder text"><b>' . PHPST_TICKET . ' #' . $ticket['ticket']['ID'] . ' ' . PHPST_INFORMATION . '</b></td>';
        if ($ticket['ticket']['status'] == 'Open') {
            $string .= '<td class="boxborder list-menu" width="30%"><a href="' . $url . '?page=viewticket&amp;action=closeticket&amp;ID=' . $ticket['ticket']['ID'] . '">' . PHPST_CLOSETICKET . '</a></td>';
        } else {
            $string .= '<td class="boxborder list-menu" width="30%"><a href="' . $url . '?page=viewticket&amp;action=openticket&amp;ID=' . $ticket['ticket']['ID'] . '">' . PHPST_REOPENTICKET . '</a></td>';
        }
        
        $string .= '
                    </tr>
                  </table>
                
                  <table width="97%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_NAME . ':</b></td>';
        if ($admin_type == 'Admin') {
            $string .= "<td class=\"boxborder text\">
                    <a href=\"$url?page=viewuser&amp;field=users.ID&amp;search={$ticket['user']['ID']}&amp;ID={$ticket['user']['ID']}\">
                    " . $ticket['user']['nickname'] . '</a></td>';
        } else {
            $string .= '<td class="boxborder text">' . $ticket['department']['name'] . '</td>';
        }
        $string .= '
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_EMAIL . ':</b></td>
                      <td class="boxborder text"><a href="mailto:' . $ticket['user']['email'] . '">' . $ticket['user']['email'] . '</a></td>
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_SUBJECT . ':</b></td>
                      <td class="boxborder text">' . $ticket['ticket']['subject'] . '</td>
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_DEPARTMENT . ':</b></td>';
        if ($admin_type == 'Admin') {
            $string .= "<td class=\"boxborder text\">
                    <a href=\"$url?page=viewdepartment&amp;field=departments.ID&amp;search={$ticket['department']['ID']}&amp;ID={$ticket['department']['ID']}\">
                    " . $ticket['department']['name'] . '</a></td>';
        } else {
            $string .= '<td class="boxborder text">' . $ticket['department']['name'] . '</td>';
        }
        $string .= '
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_URGENCY . ':</b></td>
                      <td class="boxborder text" bgcolor="#FF9966"><b>' . $ticket['ticket']['urgency'] . '</b></td>
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_STATUS . ':</b></td>
                      <td class="boxborder text">
                      <span style="color:#000000">' . $ticket['ticket']['status'] . '</span></td>
                    </tr>
                  </table>
                  
                  <div style="padding-top:5px"></div>';
            if ($ticket['ticket']['status'] == 'Open') {
                $string .='
                  <table width="97%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                    <tr bgcolor="#AABBDD">
                      <td class="boxborder text"><b>' . PHPST_RESPOND . '</b></td>
                    </tr>
                    <tr>
                      <td align="center"><textarea name="body" cols="80" rows="10"></textarea></td>
                    </tr>
                    <tr>
                      <td align="right">
                        <input type="submit" value="' . PHPST_SUBMIT .'" />
                      </td>
                    </tr>
                  </table>
                  
                  <div style="padding-top:5px"></div>
                  
                  <table width="97%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                    <tr bgcolor="#AABBDD">
                      <td class="boxborder text"><b>' . PHPST_ATTACHMENT . '</b></td>
                    </tr>
                    <tr>
                      <td class="boxborder" align="center">
                        <input type="hidden" name="MAX_FILE_SIZE" value="10240000" />
                        <input type="file"   name="userfile" size="54" />
                      </td>
                    </tr>
                  </table>
                  <br />';
            } // end if
            
            $string .= '
                </td>
                
                <td width="50%" valign="top" style="padding-top:5px">
                  
                  <table width="97%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                    <tr bgcolor="#AABBDD">
                      <td class="boxborder text">
                        <b>' . PHPST_DIALOG_QUESTION . '</b>
                      </td>
                      <td class="boxborder text" bgcolor="#AACCDD" width="50%" align="right">' . gmdate('d-m-Y H:i:s', $ticket['ticket']['timestamp']) . '</td>
                    </tr>
                    <tr>
                      <td class="boxborder text" colspan="2">' . $ticket['ticket']['body'] . '</td>
                    </tr>';
            if (isset($ticket_attachment['filename']) && $ticket_attachment['filename'] != '') {
                $string .= '
                        <tr bgcolor="#AACCEE">
                          <td class="boxborder text">' . PHPST_POSTEDBY . ': ' . $ticket['user']['nickname'] . '</td>
                          <td class="text"><b>' . @$attachment . '</b>' . @$ticket_attachment . ' <a href="' .@$link_to_file . '">' . @$download_image . '</a></td>
                        </tr>';
            }
            $string .= '</table>';
            
            foreach ($ticket['answers'] as $ID => $answer) {
                $user = User::getFromDB(array('ID' => $answer['user_ID']));
                
                // Check if there is a file attached to this answer
                unset($answer_attachment);
                $answer_attachment = array();
                $answer_attachment['path'] = PHPST_UPLOAD_PATH . $user->getUsername() 
                        . '/answer_' . $answer['ID'];
                if (is_dir($answer_attachment['path'])) {
                    // Prepare path to file and filename for linking
                    $contents = scandir($answer_attachment['path']);
                    $answer_attachment['filename'] = $contents[2];
                    $answer_attachment['link'] = PHPST_UPLOAD_RELATIVE_PATH . $user->getUsername() 
                            . '/answer_' . $answer['ID'] . '/' . $answer_attachment['filename'];
                    $answer_attachment['label'] = PHPST_ATTACHMENT;
                    $answer_attachment['image'] = '<img border="0" src="images/download.gif" />';
                }
                
                $string .= '
                     <div style="padding-top:5px"></div>
                  
                      <table width="97%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                        <tr bgcolor="#AABBDD">
                          <td class="boxborder text"><b>' . PHPST_RESPONSE . ' ' . ($ID + 1) . '</b></td>
                          <td class="boxborder text" bgcolor="#AACCDD" width="50%" align="right">' . gmdate('d-m-Y H:i:s', $answer['timestamp']) . '</td>
                        </tr>
                        <tr>
                          <td class="boxborder text" colspan="2">' . $answer['body'] . '</td>
                        </tr>
                        <tr bgcolor="#AACCEE">
                          <td class="boxborder text">' . PHPST_POSTEDBY . ': 
                            <a href="mailto:' . $user->getEmail() . '">' . $user->getNickname() . '</a>
                          </td>';
                if (isset($answer_attachment['filename']) && $answer_attachment['filename'] != '') {
                    $string .= '
                          <td class="text">
                            <b>' . @$answer_attachment['label'] . '</b>' . @$answer_attachment['filename'] . ' 
                            <a href="' .@$answer_attachment['link'] . '">' . @$answer_attachment['image'] . '</a>
                          </td>';
                }
                $string .= '
                        </tr>
                      </table> ';
            } // end foreach
                  
            $string .= '
                  <div style="padding-top:5px"></div>
                </td>
              </tr>
            </table>
            </form>';
        return $string;
    }
    
    /**
    * Builds the newticket page .
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildNewticket() {
        $url = $this->getAbsoluteUrl() . 'index.php';
        $string = $this->buildNavigation(null);
        if (is_array($this->getMsg())) {
            $msg = $this->buildErrorList();
            $this->setMsg(PHPST_TICKET_INSERT_FAILURE . $msg);
        } else {
            $this->setMsg(null);
        }
        
        $string .= '
            <script type="text/javascript" language="javascript">
            <!--
            urgency_colours = new Array(4);
            urgency_colours[0] = "#' . PHPST_URGENCY_COLOUR_1 . '";
            urgency_colours[1] = "#' . PHPST_URGENCY_COLOUR_2 . '";
            urgency_colours[2] = "#' . PHPST_URGENCY_COLOUR_3 . '";
            urgency_colours[3] = "#' . PHPST_URGENCY_COLOUR_4 . '";
            
            function updateUrgency(value) {
                document.getElementById("urgency").style.backgroundColor = urgency_colours[value - 1];
            }
            -->
            </script>
            <form enctype="multipart/form-data" action="' . $url . '" method="post">
                <input type="hidden" name="formdata" value="newticket" />
                <input type="hidden" name="page" value="newticket" />
                <input type="hidden" name="user_ID" value="' . $this->user->getID() . '" />
                <input type="hidden" name="timestamp" value="' . time() . '" />
                <input type="hidden" name="status" value="Open" />
            <table width="75%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
              <tr>
                <td class="boxborder" width="50%" valign="top" style="padding-top:5px">
                
                  <table width="97%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                    <tr>
                      <td bgcolor="#AABBDD" class="boxborder text">
                        <b>' . PHPST_NEW_SUPPORT_TICKET . '</b>
                      </td>
                    </tr>
                  </table>
                
                  <table width="97%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_NAME . ':</b></td>
                      <td class="boxborder text">
                        ' . $this->user->getNickname() . '
                      </td>
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_EMAIL . ':</b></td>
                      <td class="boxborder text">
                        ' . $this->user->getEmail() . '
                      </td>
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_SUBJECT . ':</b></td>
                      <td class="boxborder text">
                        <input type="text" name="subject" value="' . @$request['subject'] . '" />
                      </td>
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_DEPARTMENT . ':</b></td>
                      <td class="boxborder text">' . $this->getDepartmentsDropDown() . '</td>
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_URGENCY . ':</b></td>
                      <td class="boxborder text">
                        <select onchange="updateUrgency(this.value);" id="urgency" style="background-color:#' . PHPST_URGENCY_COLOUR_1 . '" name="urgency">
                            <option style="background-color:#' . PHPST_URGENCY_COLOUR_1 . '" selected="selected" value="1">' . PHPST_URGENCY_LABEL_1 . '</option>
                            <option style="background-color:#' . PHPST_URGENCY_COLOUR_2 . '" value="2">' . PHPST_URGENCY_LABEL_2 . '</option>
                            <option style="background-color:#' . PHPST_URGENCY_COLOUR_3 . '" value="3">' . PHPST_URGENCY_LABEL_3 . '</option>
                            <option style="background-color:#' . PHPST_URGENCY_COLOUR_4 . '" value="4">' . PHPST_URGENCY_LABEL_4 . '</option>
                        </select>
                      </td>
                    </tr>
                  </table>
                  
                  <div style="padding-top:5px"></div>
                  
                  <table width="97%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                    <tr bgcolor="#AABBDD">
                      <td class="boxborder text"><b>' . PHPST_QUESTION . '</b></td>
                    </tr>
                    <tr>
                      <td align="center"><textarea name="body" cols="80" rows="10"></textarea></td>
                    </tr>
                    <tr>
                      <td align="right">
                        <input type="submit" value="' . PHPST_SUBMIT .'" />
                      </td>
                    </tr>
                  </table>
                  
                  <div style="padding-top:5px"></div>
                  
                  <table width="97%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                    <tr bgcolor="#AABBDD">
                      <td class="boxborder text"><b>' . PHPST_ATTACHMENT . '</b></td>
                    </tr>
                    <tr>
                      <td class="boxborder" align="center">
                        <input type="hidden" name="MAX_FILE_SIZE" value="10240000" />
                        <input type="file"   name="userfile" size="54" />
                      </td>
                    </tr>
                  </table>
                  <br />
                  
                  <td width="50%" valign="top" style="padding-top:5px">
                  
                  <table width="97%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                    <tr>
                      <td class="text" style="text-weight:bold;">' . $this->getMsg() . '</td>
                    </tr>
                    <tr>
                       <td class="text">' . PHPST_NEWTICKET_INSTRUCTIONS . '</td>
                    </tr>
                  </table>
                </td>
              </tr>
            </table>
            </form>';
        
        return $string;
    }
    
    /**
    * Builds the createuser page.
    * 
    * @access public
    * @return string
    * @param array $request
    * @param string $admin_type
    */
    public function buildCreateuser($request, $admin_type) {
        $maintablewidth = 300;
        $maintablealign = 'center';
        $error_fields = array();
        $url = $this->getAbsoluteUrl() . 'index.php';
        
        // Add Javascript for displaying or hiding departments drop-down
        $string = '
                <script language="Javascript" type="text/javascript">
                <!--
                    function displayDepSelect(value) {
                        if (value == "Mod") {
                            document.getElementById("departments").style.display = "block";
                            document.getElementById("departments").value = 1;
                            var department = document.getElementById("department");
                            document.newuser.removeChild(department);
                        } else {
                            document.getElementById("departments").style.display = "none";
                            document.getElementById("departments").value = 0;
                            var department = document.createElement("input");
                            department.type = "hidden";
                            department.name = "department_ID";
                            department.id = "department";
                            department.value = 0;
                            document.newuser.appendChild(department);
                        }
                    }
                -->
                </script>';
        $string .= $this->buildNavigation($request);
        if (is_array($this->getMsg())) {
            $msg = $this->buildErrorList();
            $this->setMsg($msg);
        } elseif ($this->getMsg() === true) {
            // If the message equals true, it means the user was successfully added
            $this->setMsg(PHPST_USER_INSERT_SUCCESS);
            $string .= '
                    <table width="' . $maintablewidth . '" cellspacing="1" cellpadding="1" class="boxborder bodycolour" align="' . $maintablealign . '">
                      <tr><td colspan="2">' . $this->getMsg() . '</td></tr>
                    </table>';
            return $string;
        }
        
        $string .= '
            <form action="' . $url . '" method="post" name="newuser">';
        
        if ($admin_type != 'Admin') {
            $string .= '<input type="hidden" name="register" value="register" />';
        }
        
        $string .= '     
              <input type="hidden" name="formdata" value="register" />
              <input type="hidden" name="page" value="createuser" />
              <input type="hidden" name="last_login" value="' . time() . '" />
              <input type="hidden" name="last_login" value="' . time() . '" />
            <table width="' . $maintablewidth . '" cellspacing="1" cellpadding="1" class="boxborder bodycolour" align="' . $maintablealign . '">
              <tr><td colspan="2">' . $this->getMsg() . '</td></tr>
              <tr>
                <td class="boxborder text">' . PHPST_REGNAME . '</td>
                <td class="boxborder"><input class="' . eval('if (in_array("nickname", $error_fields)) return "error";') . '" name="nickname" value="' . @$request['nickname'] . '" size="35" /></td>
              </tr>
              <tr>
                <td class="boxborder text">' . PHPST_USERNAME . '</td>
                <td class="boxborder"><input class="' . eval('if (in_array("username", $error_fields)) return "error";') . '" name="username" value="' . @$request['username'] . '" size="35" /></td>
              </tr>
              <tr>
                <td class="boxborder text">' . PHPST_PASSWORD . '</td>
                <td class="boxborder"><input class="' . eval('if (in_array("password", $error_fields)) return "error";') . '" type="password" value="' . @$request['password'] . '" name="password" size="35" /></td>
              </tr>
              <tr>
                <td width="100%" class="boxborder text">' . PHPST_EMAIL . '</td>
                <td class="boxborder"><input  class="' . eval('if (in_array("email", $error_fields)) return "error";') . '" value="' . @$request['email'] . '" name="email" size="35" /></td>
              </tr>';
        
        if ($admin_type == 'Admin') {
            $string .= '
                  <tr>
                    <td width="100%" class="boxborder text">' . PHPST_DEPARTMENT . '</td>
                    <td class="boxborder">
                        ' . $this->getDepartmentsDropDown($request) . '
                    </td>
                    <script language="JavaScript" type="text/javascript">
                    <!--
                        displayDepSelect("' . @$request['admin_status'] . '");
                    -->
                    </script>
                  </tr>
                  <tr>
                      <td class="text" align="center" colspan="2">
                    	' . PHPST_ADMIN_STATUS . '<br />
                        ' . PHPST_CLIENT . ': <input onclick="displayDepSelect(this.value)" type="radio" ' . eval('if (@$request["admin_status"] == "Client") return \'checked="checked"\';') . ' name="admin_status" value="Client" />
                    	' . PHPST_MOD . ': <input onclick="displayDepSelect(this.value)" type="radio" ' . eval('if (@$request["admin_status"] == "Mod") return \'checked="checked"\';') . ' name="admin_status" value="Mod" />
                    	' . PHPST_ADMIN . ': <input onclick="displayDepSelect(this.value)" type="radio" ' . eval('if (@$request["admin_status"] == "Admin") return \'checked="checked"\';') . ' name="admin_status" value="Admin" />
                      </td>
                  </tr>';
        } else {
            $string .= '<input type="hidden" name="admin_status" value="Client" />
                    <input type="hidden" name="department_ID" value="0" />';
        }
        
        $string .= '
            </table>
            
            <table width="300" cellspacing="0" cellpadding="2" align="center">
              <tr>
                <td align="right"><input type="submit" value="Submit" /></td>
              </tr>
            </table>
            </form>';
			return $string;
    }
    
    /**
    * Builds a list of errors.
    * 
    * @access public
    * @return string
    */
    public function buildErrorList() {
        $errors = $this->getMsg();
        $msg = "<ul>";
        foreach ($errors as $error) {
            
            $msg .= "<li>{$error['message']}</li>";
            $error_fields[] = $error['field'];
        }
        $msg .= "</ul>";
        return $msg;
    }
    
    /**
    * Builds the search box HTML.
    * 
    * @access public
    * @param string $admin_type
    * @param string $searchterm
    * @return string
    */
    public function buildSearchBox($searchterm = 'tickets.subject', $page = 'home') {
        $terms = explode('.', $searchterm);
        $field = eval('return ' . 'PHPST_' . strtoupper($terms[0]) . ';');
        $url = $this->getAbsoluteUrl() . 'index.php';
        $string = '
                <form action="' . $url . '?page=' . $page . '" method="post">
                <table width="75%" cellspacing="1" cellpadding="1" align="center">
                  <tr>
                    <td>
                      <a href="' . $url . '" >
                        <img src="images/support_tickets_logo.gif" width="83" height="61" title="Triangle Solutions PHP Support Tickets" alt="Triangle Solutions PHP Support Tickets" vspace="2" border="0" />
                      </a>
                    </td>
                    <td>
                      <table>
                        <tr>
                          <td class="text" style="padding:2px">' . PHPST_CURRENT_PAGE . $this->getPageName() . '</td>
                        </tr>
                        <tr>
                          <td class="text" style="padding:2px">' . PHPST_LOGGEDAS . ' ' . $this->user->getNickname() . ' (' . $this->user->getAdmin_status() . ')</td>
                        </tr>
                      </table>
                    </td>
                    <td valign="bottom" align="right" class="text" style="padding-bottom:5px">' . PHPST_SEARCH . ' ' . $field .':
                      <input type="hidden" name="field" value="' . $searchterm . '" />
                      <input name="search" size="24" />
                      <input type="submit" value="' . PHPST_GO . '" />
                    </td>
                  </tr>
                </table>
                </form>';
        return $string;
    }
    
    /**
    * Builds the navigation HTML.
    * 
    * @access public
    * @param array $request
    * @return string
    */
    public function buildNavigation($request) {
        $url = $this->getAbsoluteUrl() . 'index.php';
        $string = $this->buildSearchBox();
        $string .= '
                <table width="75%" cellspacing="1" cellpadding="1" class="boxborder titlebarcolour" align="center">
                  <tr>
                    <td class="boxborder text"><a href="' . $url . '" title="' . PHPST_STTITLE . '">' . PHPST_TITLELINK .'</a> - ' . PHPST_STTITLE . '</td>
                    <td class="boxborder list-menu" width="15%"><a href="' . $url . '?page=newticket" title="' . PHPST_TITLEREQ . '">' . PHPST_TITLEREQ . '</a></td>
                    <td class="boxborder list-menu" width="15%"><a href="' . $url . '?field=status&amp;search=open" title="' . PHPST_TITLEOPE . '">' . PHPST_TITLEOPE . '</a></td>
                    <td class="boxborder list-menu" width="15%"><a href="' . $url . '?field=status&amp;search=closed" title="' . PHPST_TITLECLO . '">' . PHPST_TITLECLO . '</a></td>
                    <td class="boxborder list-menu" width="10%"><a href="' . $url . '?logout" title="' . PHPST_TITLELOG . '">' . PHPST_TITLELOG . '</a></td>
                  </tr>
                </table>
                ';
        return $string;
    }
    
    /**
     * Adds the footer to the GUI's HTML.
     * 
     * @access public
     * @return void
     **/
    public function buildFooter() {
        $string = '<table width="100%" cellspacing="1" cellpadding="1" align="center">
        	  <tr>
        		<td align="center" class="text">
        		</td>
        	  </tr>
        	  <tr>
        		<td align="center"><br />
        		<a href="http://www.triangle-solutions.com" target="_blank" title="triangle solutions web development">Triangle Solutions Ltd</a> |
        		<a href="http://www.phpsupporttickets.com" target="_blank" title="php support tickets">PHP Support Tickets v2.0</a><br /><br />
        		</td>
        	  </tr>
        	</table>
        
        	</body>
        	</html>';
        $this->addHTML($string);
    }
    
    /**
    * Builds the browseticket page.
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildBrowsetickets($request, $tickets) {
        $url = $this->getAbsoluteUrl() . 'index.php';
        
        $numberOfPages = ceil($tickets['count'] / PHPST_MAX_RECORDS);
        $search = 'all';
        $new_order = 'ASC';
        
        if (isset($request['sort']) && isset($request['order'])) {
            $sort = $request['sort'];
            $order = $request['order'];
            if ($order == 'ASC') {
                $new_order = 'DESC';
            } else {
                $new_order = 'ASC';
            }
        }
        
        if (isset($request['search'])) {
            $search = $request['search'];
        }
        
        $string = $this->buildNavigation($request);
        
        $recent_tickets = 0;
        if ($tickets != false) {
            foreach ($tickets as $t) {
                if ($t['recent']) $recent_tickets++;
            }
            $string .= $this->buildPagination($numberOfPages, $request, $tickets['count']);
            $string .= '
                <table width="75%" cellspacing="1" cellpadding="1" class="boxborder titlebarcolour" align="center">
                  <tr>
                    <td class="boxborder text">' . PHPST_RECENT . ': ' . $recent_tickets . ' - ' . PHPST_CLICK . '.</td>
                  </tr>
                </table>
                <script language="javascript" type="text/javascript">
                <!--
                function check_all() {
                  for (var c = 0; c < document.myform.elements.length; c++) {
                    if (document.myform.elements[c].type == \'checkbox\') {
                      if(document.myform.elements[c].checked == true) {
                        document.myform.elements[c].checked = false;
                      } else {
                        document.myform.elements[c].checked = true;
                      }
                    }
                  }
                }
                // -->
                </script>
                
                <form name="myform" action="' . $url . '" method="post">
                <input type="hidden" name="formdata" value="changetickets" />
                <input type="hidden" name="page" value="browsetickets" />
                <table width="75%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                  <tr align="center" class="sectionscolour">
                    <td class="boxborder text" onclick="check_all();" style="cursor:pointer">
                      <b><u>' . PHPST_ALL . '</u></b>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?field=status&amp;ticket_type=' . @$request['ticket_type'] . '&amp;search=' . $search . '&amp;sort=ID&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_TICKETID . '">
                        <b>' . PHPST_TICKETID . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="#" title="' . PHPST_ORDERBY . ' ' . PHPST_REPLIES . '">
                        <b>' . PHPST_REPLIES . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?field=status&amp;ticket_type=' . @$request['ticket_type'] . '&amp;search=' . $search . '&amp;sort=subject&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_SUBJECT . '">
                        <b>' . PHPST_SUBJECT . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?field=status&amp;ticket_type=' . @$request['ticket_type'] . '&amp;search=' . $search . '&amp;sort=timestamp&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_TIME . '">
                        <b>' . PHPST_TIME . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?field=status&amp;ticket_type=' . @$request['ticket_type'] . '&amp;search=' . $search . '&amp;sort=urgency&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_URGENCY . '">
                        <b>' . PHPST_URGENCY . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?field=status&amp;ticket_type=' . @$request['ticket_type'] . '&amp;search=' . $search . '&amp;sort=department_name&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_DEPARTMENT . '">
                        <b>' . PHPST_DEPARTMENT . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?field=status&amp;ticket_type=' . @$request['ticket_type'] . '&amp;search=' . $search . '&amp;sort=status&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_STATUS . '">
                        <b>' . PHPST_STATUS . '</b>
                      </a>
                    </td>
                  </tr>';
              
              foreach ($tickets as $key => $v) {
                  if ($key != 'count') {
                      $ticket = ($v['ticket']);
                      
                      // Prepare class for highlighting ticket if recent
                      $recent = '';
                      if ($v['recent']) {
                          $recent = 'recent';
                      } 
                      
                      $string .= '
                      <tr align="center" bgcolor="#F4FAFF">
                        <td class="boxborder"><input type="checkbox" name="ticket[]" value="' . $ticket['ID'] . '" /></td>
                        <td class="boxborder list-menu">
                          <a href="' . $url . '?page=viewticket&amp;ID=' . $ticket['ID'] . '">' . $ticket['ID'] . '</a>
                        </td>
                        <td class="boxborder text">[' . @count($v['answers']) . ']</td>
                        <td class="boxborder text">' . $ticket['subject'] . '</td>
                        <td class="boxborder text ' . @$recent . '">' . gmdate('Y m d H:i:s',$ticket['timestamp']) . '</td>
                        <td class="boxborder text" bgcolor="#' . eval('return PHPST_URGENCY_COLOUR_' . $ticket['urgency'] . ';') . '">
                          ' . eval('return PHPST_URGENCY_LABEL_' . $ticket['urgency'] . ';') . '
                        </td>
                        <td class="boxborder text">' . $v['department']['name'] . '</td>
                        <td class="boxborder text">
                        <span style="color:#000000">' . $ticket['status'] . '</span></td>
                      </tr>';
                  }
              }
                  
                  $string .= '
                          <tr class="sectionscolour">
                            <td colspan="8">
                              <select name="status">
                              <option value="open">' . PHPST_OPEN . '</option>
                              <option value="closed">' . PHPST_CLOSED . '</option>
                              </select>
                              <input type="submit" name="sub" value="' . PHPST_GO . '" />
                            </td>
                          </tr>
                        </table>
                        </form>
                        ';
                  $string .= $this->buildPagination($numberOfPages, $request, $tickets['count']);
        } elseif ($search != 'all') {
            $string .= '
                <div style="padding-top:5px"></div>
                <table width="75%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                  <tr>
                    <td class="text">' . PHPST_NO_RESULTS . '</td>
                  </tr>
                </table>';
        } else {
            $string .= '
                <div style="padding-top:5px"></div>
                <table width="75%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                  <tr>
                    <td class="text">' . PHPST_NO_TICKETS . '</td>
                  </tr>
                </table>';
        }
        return $string;
    }
    
    /**
    * Builds the HTML selection title.
    * 
    * @access public
    * @param array $request
    * @return string html
    */
    public function buildSelectionTitle($request) {
        $retString = '';
        if (isset($request['field'])) {
            $retString = PHPST_SELECTION . ucfirst($request['field']) . ' = ' . $request['search'];
        }
        
        return $retString;
    }
    
    /**
    * Builds HTML pagination system based on page number and record Count.
    * 
    * @access public
    * @param int $numberOfPages
    * @param array $request
    * @param int $count The total number or records
    * @return string HTML
    */
    public function buildPagination($numberOfPages, $request, $count = 0) {
        
        $url = $this->getAbsoluteUrl() . 'index.php';
        $string = '
            <div style="padding-top:5px"></div>
            <table width="75%" cellspacing="1" cellpadding="1" class="boxborder pagingcolour" align="center">
              <tr valign="middle" align="center">';
        
        // Previous Link
        if (isset($request['rs_page']) && $request['rs_page'] > 1) {
            $string .= '
                <td class="boxborder text" width="75">
                  <a href="' . $url . '?page=' . @$request['page'] . '&amp;order=' . @$request['order'] . '&amp;sort=' . @$request['sort'] . '&amp;search=' . @$request['search'] . '&amp;field=' . @$request['field'] . '&amp;rs_page=' . ($request['rs_page'] - 1) . '">&#171;&nbsp;' . PHPST_PREVIOUS . '</a>
                </td>';
        } else {
            $request['rs_page'] = 1;
            $string .= '
                <td class="boxborder text" width="75">
                </td>';
        }
        
        // Page links
        $string .= '
            <td class="boxborder text">
                <b>' . PHPST_PAGES . '</b>';
        
        for ($i = 1; $i <= $numberOfPages; $i++) {
            if ($i == $request['rs_page']) {
                $string .= " <span class=\"selected\">[$i]</span> ";
            } else {
                $string .= ' <a href="' . $url . '?page=' . @$request['page'] . '&amp;order=' . @$request['order'] . '&amp;sort=' . @$request['sort'] . '&amp;search=' . @$request['search'] . '&amp;field=' . @$request['field'] . '&amp;rs_page=' . $i . '">[' . $i . ']</a> ';
            }
        }
        
        // Correct plural grammar in English. Comment out for other languages where this doesn't apply
        $records = PHPST_RECORDS;
        if ($count == 1) {
            $records = PHPST_RECORD;
        }
        
        $string .= " <span class=\"totalrecords\">($count $records)</span></td>";
        
        // Next link
        if ($request['rs_page'] < $numberOfPages) {
            $string .= '
                <td class="boxborder text" width="75">
                  <a href="' . $url . '?page=' . @$request['page'] . '&amp;order=' . @$request['order'] . '&amp;sort=' . @$request['sort'] . '&amp;search=' . @$request['search'] . '&amp;field=' . @$request['field'] . '&amp;rs_page=' . ($request['rs_page'] + 1) . '">' . PHPST_NEXT . '&nbsp;&#187;</a>
                </td>';
        } else {
            $string .= '
                <td class="boxborder text" width="75">
                </td>';
        }
        $string .= '
                  </tr>
                </table>';
         return $string;
    }
    
    /**
    * Reset's this GUI's HTML to an empty string.
    * 
    * @access public
    * @return void
    */
    public function resetHTML() {
        $this->html = '';
    }   
    
    /**
    * Returns this GUI's page variable.
    * 
    * @access public
    * @return string
    */
    public function getPage() {
        return $this->page;
    }
    
    /**
    * Returns this GUI's page variable, formatted according to the language file definition.
    * 
    * @access public
    * @return string
    */
    public function getPageName() {
        return eval('return PHPST_PAGENAME_' . strtoupper($this->page) . ';');
    }
    
    /**
    * Sets this GUI's page variable.
    * 
    * @access public
    * @param string $string
    * @return void
    */
    public function setPage($string) {
        $this->page = $string;
    }
    
    /**
    * Returns this GUI's msg variable.
    * 
    * @access public
    * @return string
    */
    public function getMsg() {
        return $this->msg;
    }
    
    /**
    * Sets this GUI's msg variable.
    * 
    * @access public
    * @param string $string
    * @return void
    */
    public function setMsg($string) {
        $this->msg = $string;
    }
    
    /**
    * Builds a drop-down menu of all available departments
    * 
    * @access public
    * @param array $request
    * @param string $selected The name of the selected department
    * @return string
    */ 
    public function getDepartmentsDropDown($request = null, $selected = null) {
        $departments = FormHandler::get_departments(array('rs_page' => 'all'));
        $string = '<select onload="displayDepSelect(\'' . @$request['admin_status'] . '\')" id="departments" name="department_ID">';
        
        foreach ($departments as $key => $dep) {
            if ($dep != '' && !is_null($dep) && $dep['department']['name'] != '') {
                $string .= '<option';
                if (isset($selected) && $selected == $dep['department']['name']) {
                    $string .= ' selected="selected" ';
                }
                $string .= ' value="' . $dep['department']['ID'] . '">' . $dep['department']['name'] . '</option>';
            }
        }
        $string .= '</select>';
        return $string;
    }
    
    /**
    * Takes the REQUEST variables in order to change the GUI's state.
    * 
    * @access public
    * @param array $request
    * @param array $files (optional)
    * @return void
    */
    public function update($request, $files = null) {
        $this->setMsg('');
        $this->resetHTML();
        
        // Handle request data and update object
        
        // Set the page name
        if (isset($request['page']) && $request['page'] != 'login') {
            $this->setPage($request['page']);
        } else {
            $this->setPage('home');
        }
        
        // Process user data entry
        if (isset($request['formdata'])) {
            $this->setMsg(FormHandler::submitForm($this->getPage(), $request['formdata'], 
                    $request, $files));
        }
        
        // If form was submitted and passed validation, redirect user
        if (isset($request['formdata']) && !is_array($this->getMsg())) {
            switch ($request['page']) {
                case 'newticket':
                    // @todo redirect to the ticket just created, viewticket page
                    $this->setPage('home');
                    break;
                case 'newuser':
                    // @todo redirect to the user just created, viewuser page
                    $this->setPage('home');
                    break;
                default:
                    break;
            }
        }
        $this->buildHeader($this->getPageName());
        $this->buildBody($request);
        $this->buildFooter();

    }
    
    /**
    * Builds the viewdepartment page.
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildViewdepartment($request) {
        $url = $this->getAbsoluteUrl() . 'index.php';
        $department = FormHandler::get_department($request);
        $admin_type = $this->user->getAdmin_status();
        $field = '';
        if (is_array($this->getMsg())) {
            $msg = $this->getMsg();
            $error = $msg[0]['field'];
            $this->setMsg($msg[0]['message']);
        } 
        
        $string = $this->buildNavigation($request);
        if ($admin_type == 'Admin') {
            $string .= '
            <form enctype="multipart/form-data" action="' . $url . '" method="post">
                <input type="hidden" name="formdata" value="editdepartment" />
                <input type="hidden" name="page" value="viewdepartment" />
                <input type="hidden" name="field" value="department.ID" />
                <input type="hidden" name="ID" value="' . $department['department']['ID'] . '" />
                <input type="hidden" name="search" value="' . $department['department']['ID'] . '" />';
        }
        
        $string .= '
            <table width="75%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
              <tr>
                <td class="boxborder" width="50%" valign="top" style="padding-top:5px">
                
                  <table width="97%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_NAME . ':</b>
                      </td>
                      <td class="boxborder text">';
        if ($admin_type == 'Admin') {
            $string .= '<input type="text" name="name" size="50" value="' . $department['department']['name'] . '" />';
        } elseif ($admin_type == 'Mod') {
            $string .= $department['department']['name'];
        }
        
        $string .= ' 
                      </td>
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_DESCRIPTION . ':</b></td>
                      <td class="boxborder text">';
        if ($admin_type == 'Admin') {
            $string .= '<textarea name="description" cols="45" rows="6">'
                           . $department['department']['description'] .
                        '</textarea>';
        } elseif ($admin_type == 'Mod') {
            $string .= $department['department']['description'];
        }
        
        $string .= '
                      </td>
                    </tr>';
        if ($admin_type == 'Admin') {
            $deleteUrl = $url . '?page=deletedepartment&amp;ID=' . $department['department']['ID'];
            $string .= '<tr>
                          <td align="right">
                            <input type="submit" value="' . PHPST_SUBMIT .'" />
                            <button onclick="window.location=\'' . $deleteUrl . '\'">' . PHPST_DELETE . '</button>
                          </td>
                        </tr>';
        } 
        $string .= '
                    
                  </table>
                  
                  <div style="padding-top:5px"></div>
                  
                  <br />
                </td>
                
                <td width="50%" valign="top" style="padding-top:5px">
                  <table width="98%" cellspacing="1" cellpadding="1" class="boxborder" align="center">';
        if (isset($department['tickets'])) {
            // Remove closed tickets
            foreach ($department['tickets'] as $ID => $ticket) {
                if ($ticket['status'] != 'Open') {
                    unset($department['tickets'][$ID]);
                }
            }
        }
        
        // If any tickets left open, display them
        if (@count($department['tickets']) > 0) {
        
            $string .= '
                  <tr bgcolor="#AABBDD">
                    <td colspan="6" class="boxborder text"><b>' . PHPST_TICKETS . '</b></td>
                  </tr>
                  <tr align="center" class="sectionscolour">
                    <td class="boxborder text"><b>' . PHPST_TICKETID . '</b></td>
                    <td class="boxborder text"><b>' . PHPST_REPLIES . '</b></td>
                    <td class="boxborder text"><b>' . PHPST_SUBJECT . '</b></td>
                    <td class="boxborder text"><b>' . PHPST_TIME . '</b></td>
                    <td class="boxborder text"><b>' . PHPST_URGENCY . '</b></td>
                    <td class="boxborder text"><b>' . PHPST_STATUS . '</b></td>
                  </tr>';
          
            foreach ($department['tickets'] as $ID => $ticket) {
                $myTicket = FormHandler::get_ticket($ticket);
                $answers = @count($myTicket['answers']);
                $string .= '
                      <tr align="center" bgcolor="#F4FAFF">
                        <td class="boxborder list-menu"><a href="' . $url . '?page=viewticket&amp;ID=' . $ticket['ID'] . '">' . $ticket['ID'] . '</a></td>
                        <td class="boxborder text">[' . $answers . ']</td>
                        <td class="boxborder text">' . $ticket['subject'] . '</td>
                        <td class="boxborder text">' . gmdate('Y m d H:i:s',$ticket['timestamp']) . '</td>
                        <td class="boxborder text" bgcolor="#' . eval('return PHPST_URGENCY_COLOUR_' . $ticket['urgency'] . ';') . '">' . eval('return PHPST_URGENCY_LABEL_' . $ticket['urgency'] . ';') . '</td>
                        <td class="boxborder text">
                        <span style="color:#000000">' . $ticket['status'] . '</span></td>
                      </tr>';
            } // end foreach
        } else {
            $string .= '
                  <tr bgcolor="#AABBDD">
                    <td colspan="4" class="boxborder text"><b>' . PHPST_NO_TICKETS . '</b></td>
                  </tr>';
        }
        
        $string .= '
                </table>
              <div style="padding-top:5px"></div>
              <table width="98%" cellspacing="1" cellpadding="1" class="boxborder" align="center">';
        if (isset($department['users'])) {
              $string .= '
                  <tr bgcolor="#AABBDD">
                    <td colspan="4" class="boxborder text"><b>' . PHPST_MODS . '</b></td>
                  </tr>
                  <tr align="center" class="sectionscolour">
                    <td class="boxborder text"><b>' . PHPST_USERID . '</b></td>
                    <td class="boxborder text"><b>' . PHPST_NICKNAME . '</b></td>
                    <td class="boxborder text"><b>' . PHPST_USERNAME . '</b></td>
                    <td class="boxborder text"><b>' . PHPST_EMAIL . '</b></td>
                  </tr>';
            foreach ($department['users'] as $ID => $user) {
                $myUser = FormHandler::get_user($user);
                $string .= '<tr align="center" bgcolor="#F4FAFF">';
                if ($admin_type == 'Admin') {
                    $string .= '
                            <td class="boxborder list-menu"><a href="' . $url . '?page=viewuser&amp;ID=' . $user['ID'] . '">' . $user['ID'] . '</a></td>';
                } elseif ($admin_type == 'Mod') {
                    $string .= '
                            <td class="boxborder list-menu">' . $user['ID'] . '</td>';
                }
                $string .= '<td class="boxborder text">' . $user['nickname'] . '</td>
                            <td class="boxborder text">' . $user['username'] . '</td>
                            <td class="boxborder text"><a href="mailto:' . $user['email'] . '">' . $user['email'] . '</a></td>
                          </tr>';
            }
        } else {
            $string .= '
                  <tr bgcolor="#AABBDD">
                    <td colspan="4" class="boxborder text"><b>' . PHPST_NO_MODS . '</b></td>
                  </tr>';
        }
        $string .= '
                  </table>
                </td>
              </tr>
            </table>
            </form>';
        return $string;
    }
    
    /**
    * Abstract method for all GUI implementations.
    * Builds the body HTML based on the GUI's current state.
    * 
    * @abstract
    * @access public
    * @return string
    * @param array $request
    */
    public abstract function buildBody($request);
    
    /**
    * Abstract method for all GUI implementations.
    * Redirects the GUI to the page set as the home page.
    * 
    * @abstract
    * @access public
    * @return string
    * @param array $request
    */
    public abstract function buildHome($request);
} 
?>
