<?php
/**
 * File containing the AdminGUI class.
 * 
 * @category Class
 * @package PHPSupportTickets
 * @author Ian Warner, <iwarner@triangle-solutions.com> 
 * @author Nicolas Connault, <nicou@sweetpeadesigns.com.au> 
 * @copyright (C) 2001 Triangle Solutions Ltd
 * @version CVS: $Id: class.AdminGUI.php,v 1.21 2005/09/02 03:12:47 nicconn Exp $
 * @since File available since Release 1.1.1.1
 * \\||
 */

/**
* user defined includes.
* @include
*/
require_once('abstract.GUI.php');

/**
 * AdminGUI is the Admin's User Interface class.
 * 
 * @access public
 * @package PHPSupportTicket
 */
class AdminGUI extends GUI{
    /// --- FIELDS ---
    
    // --- METHODS ---
    /**
    * Constructor method.
    * 
    * @access public
    * @param string $absoluteURL
    * @param array $request
    */
    public function __construct($absoluteURL, $request) {
        parent::__construct($absoluteURL, $request);
        $this->buildHeader();
        $this->buildBody($request);
        $this->buildFooter();
    }
    
    /**
    * Updates this GUI using the $_REQUEST data sent from the previous page.
    * 
    * @access public
    * @param array $request
    * @param array $files (optional)
    * @return void
    */
    public function update($request, $files = null) {
        parent::update($request, $files);
    }
    
    /**
    * Builds the body of the GUI based on its current state.
    * 
    * @access public
    * @return void
    * @param array $request
    */
    public function buildBody($request) {
        if ($this->getPage() == '') {
            $this->setPage("home");
        }
        
        // Dynamically prepare the build function matching the requested page.
        $method = "build" . ucfirst($this->getPage());
        if (in_array($method, get_class_methods($this))) {
            $this->addHTML($this->$method($request));
        } else {
            $this->addHTML($this->buildHome($request));
        }
    }
    
    /**
    * Builds the createuser page.
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildCreateuser($request) {
        return parent::buildCreateuser($request, 'Admin');
    }
    
    /**
    * Builds the createdepartment page.
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildCreatedepartment($request) {
        $maintablewidth = 300;
        $maintablealign = 'center';
        $error_fields = array();
        $url = $this->getAbsoluteUrl() . 'index.php';
        $string = $this->buildNavigation($request);
        
        if (is_array($this->getMsg())) {
            $msg = $this->buildErrorList();
            $this->setMsg($msg);
        } elseif ($this->getMsg() === true) {
            // If the message equals true, it means the department was successfully added
            $this->setMsg(PHPST_DEPARTMENT_INSERT_SUCCESS);
            $string .= '
                    <table width="' . $maintablewidth . '" cellspacing="1" cellpadding="1" class="boxborder bodycolour" align="' . $maintablealign . '">
                      <tr><td colspan="2">' . $this->getMsg() . '</td></tr>
                    </table>';
            return $string;
        }
        
        $string .= '
            <form action="' . $url . '" method="post" name="newdepartment">     
              <input type="hidden" name="formdata" value="newdepartment" />
              <input type="hidden" name="page" value="createdepartment" />
            <table width="' . $maintablewidth . '" cellspacing="1" cellpadding="1" class="boxborder bodycolour" align="' . $maintablealign . '">
              <tr><td colspan="2">' . $this->getMsg() . '</td></tr>
              <tr>
                <td class="boxborder text">' . PHPST_NAME . '</td>
                <td class="boxborder"><input class="' . eval('if (in_array("name", $error_fields)) return "error";') . '" name="name" value="' . @$request['name'] . '" size="35" /></td>
              </tr>
              <tr>
                <td class="boxborder text">' . PHPST_DESCRIPTION . '</td>
                <td class="boxborder"><textarea class="' . eval('if (in_array("description", $error_fields)) return "error";') . '" name="description" cols="40" rows="4">' . @$request['username'] . '</textarea></td>
              </tr>
            </table>
            
            <table width="300" cellspacing="0" cellpadding="2" align="center">
              <tr>
                <td align="right"><input type="submit" value="Submit" /></td>
              </tr>
            </table>
            </form>';
			return $string;
    }
    
    /**
    * Builds the home page (browse tickets).
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildBrowsetickets($request) {
        // Retrieve list of tickets from DB
        $tickets = FormHandler::get_tickets('Admin', $request , null);
        return parent::buildBrowsetickets($request, $tickets);
    }
    
    /**
    * Redirects to this GUI's home page.
    * 
    * @access public
    * @return void
    * @param array $request
    */
    public function buildHome($request) {
        return $this->buildBrowsetickets($request);
    }
    
    /**
    * Builds the viewticket page (view tickets).
    * 
    * @access public
    * @return void
    * @param array $request
    */
    public function buildViewticket($request) {
        // Retrieve ticket info from DB
        // Note this is not a Ticket object, but an array containing a Ticket, a Department
        // and a User objects associated with that Ticket.
        $ticket = FormHandler::get_ticket($request);
        return parent::buildViewticket($request, $ticket);
    }
    
    /**
    * Builds the navigation HTML.
    *  
    * @access public
    * @param array $request
    * @param string $searchterm
    * @param string $page
    * @return string
    */
    public function buildNavigation($request, $searchterm = 'tickets.subject', $page = 'home') {
        $string = parent::buildSearchBox($searchterm, $page);
        $selection_title = parent::buildSelectionTitle($request);
        $url = $this->getAbsoluteUrl() . 'index.php';
        $string .= '
                <script language="JavaScript" type="text/javascript">
                <!--
                function showMenu(value) {
                    hideMenu("userspan");
                    hideMenu("departmentspan");
                    hideMenu("ticketspan");
                    hideMenu("optionspan");
                    document.getElementById(value).style.display = "block";
                }

                function hideMenu(value) {
                    if (value == "all") {
                        hideMenu("userspan");
                        hideMenu("departmentspan");
                        hideMenu("ticketspan");
                        hideMenu("optionspan");
                    } else {
                        document.getElementById(value).style.display = "none";
                    }
                }
                -->
                </script>
                <table width="75%" cellspacing="1" cellpadding="1" class="boxborder titlebarcolour" align="center">
                  <tr>
                    <td class="boxborder text"><a href="' . $url . '" title="' . PHPST_STTITLE . '">' . PHPST_TITLELINK .'</a> - ' . PHPST_STTITLE . '</td>
                    <td id="usermenu" class="boxborder list-menu">
                      <a href="' . $url . '?page=browseusers" onmouseover="showMenu(\'userspan\');">' . PHPST_USERS . '</a>
                    </td>
                    <td id="departmentmenu" class="boxborder list-menu">
                      <a href="' . $url . '?page=browsedepartments" onmouseover="showMenu(\'departmentspan\');">' . PHPST_DEPARTMENTS . '</a>
                    </td>
                    <td id="ticketmenu" class="boxborder list-menu">
                      <a href="' . $url . '?page=browsetickets" onmouseover="showMenu(\'ticketspan\');">' . PHPST_TICKETS . '</a>
                    </td>
                    <td id="optionmenu" class="boxborder list-menu">
                      <a href="' . $url . '?page=options" onmouseover="hideMenu(\'all\');">' . PHPST_OPTIONS . '</a>
                    </td>
                    <td id="logoutmenu" class="boxborder list-menu" onmouseover="hideMenu(\'all\');">
                      <a href="' . $url . '?logout">' . PHPST_TITLELOG . '</a>
                    </td>
                  </tr>
                  <tr>
                    <td style="text-align: left; height: 25px;">' . $selection_title . '</td>
                    <td width="50%" colspan="5" style="text-align: right; height: 25px;">
                      <span id="userspan" class="submenu">
                        <a href="' . $url . '?page=browseusers" title="' . PHPST_BROWSE . '">' . PHPST_BROWSE .'</a>
                        <a href="' . $url . '?page=createuser" title="' . PHPST_CREATE_USER . '">' . PHPST_CREATE_USER .'</a>
                        <a href="' . $url . '?page=contactuser" title="' . PHPST_CONTACT_USER . '">' . PHPST_CONTACT_USER .'</a>
                        <a href="' . $url . '?page=myaccount" title="' . PHPST_MYACCOUNT . '">' . PHPST_MYACCOUNT .'</a>
                      </span>
                      <span id="departmentspan" class="submenu">
                        <a href="' . $url . '?page=browsedepartments" title="' . PHPST_BROWSE . '">' . PHPST_BROWSE .'</a>
                        <a href="' . $url . '?page=createdepartment" title="' . PHPST_CREATE_DEPARTMENT . '">' . PHPST_CREATE_DEPARTMENT .'</a>
                      </span>
                      <span id="ticketspan" class="submenu">
                        <a href="' . $url . '?page=newticket" title="' . PHPST_TITLEREQ . '">' . PHPST_TITLEREQ .'</a>
                        <a href="' . $url . '?field=status&amp;search=all" title="' . PHPST_BROWSE . '">' . PHPST_BROWSE .'</a>
                        <a href="' . $url . '?field=status&amp;search=open" title="' . PHPST_TITLEOPE . '">' . PHPST_TITLEOPE .'</a>
                        <a href="' . $url . '?field=status&amp;search=closed" title="' . PHPST_TITLECLO . '">' . PHPST_TITLECLO .'</a>
                      </span>
                      <span id="optionspan" class="submenu">
                        <a href="' . $url . '?page=emailoptions" title="' . PHPST_EMAILOPT . '">' . PHPST_EMAILOPT .'</a>
                        <a href="' . $url . '?page=privileges" title="' . PHPST_PRIVILEGES . '">' . PHPST_PRIVILEGES .'</a>
                        <a href="' . $url . '?page=ratings" title="' . PHPST_RATINGS . '">' . PHPST_RATINGS .'</a>
                        <a href="' . $url . '?page=files" title="' . PHPST_FILES . '">' . PHPST_FILES .'</a>
                        <a href="' . $url . '?page=privacy" title="' . PHPST_PRIVACY . '">' . PHPST_PRIVACY .'</a>
                      </span>
                    </td>
                  </tr>
                </table>';
        return $string;
    }
    
    /**
    * Builds the options page
    * 
    * @access public
    * @return string HTML
    * @param array $request
    */
    public function buildOptions($request) {
        $url = $this->getAbsoluteUrl() . 'index.php';
        
        // Load options from DB
        $options = FormHandler::get_options();
        
        // Build table and form headers
        $string = $this->buildNavigation($request);
        $string .= '<form name="editoption" action="' . $url . '" method="post">
                    <input type="hidden" name="formdata" value="edit" />
                    <input type="hidden" name="page" value="options" />
                    <table width="75%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                    ';
        // Display options dynamically in rows
        foreach ($options as $option) {
            // Prepare input field
            $input = '';
            switch ($option['type']) {
                case 'boolean':
                    $selected_0 = '';
                    $selected_1 = '';
                    if ($option['value'] == 0) {
                        $selected_0 = ' checked="checked" ';
                    } else {
                        $selected_1 = ' checked="checked" ';
                    }
                    $input = PHPST_YES . '<input type="radio"' . $selected_1 . ' name="' . $option['name'] . '" value="1" /> '
                            . PHPST_NO . '<input type="radio"' . $selected_0 . ' name="' . $option['name'] . '" value="0" />';
                    break;
                case 'integer':
                    $input = '<input type="text" name="' . $option['name'] . '" value="' . $option['value'] . '" />';
                    break;
                default:
                    break;
            }
            
            // Add row           
            $string .= '
                  <tr align="left">
                    <td class="boxborder text sectionscolour">' . eval('return PHPST_OPTION_' . $option['ID'] . ';') . '</td>
                    <td class="boxborder text">' . $input . '</td>
                  </tr>';
        }
        
        // Close table and form
        $string .= '
                  <tr align="right">
                    <td class="boxborder text" colspan="2">
                      <input type="submit" value="' . PHPST_SUBMIT .'" />
                    </td>
                  </tr>
            </table>
            </form>';
        
        return $string;
    }
    
    /**
    * Builds the browseusers page.
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildBrowseusers($request) {
        
        // Retrieve list of users from DB
        $users = FormHandler::get_users($request);
        
        $url = $this->getAbsoluteUrl() . 'index.php';
        $numberOfPages = ceil($users['count'] / PHPST_MAX_RECORDS);
        $search = 'all';
        $new_order = 'ASC';
        
        if (isset($request['sort']) && isset($request['order'])) {
            $sort = $request['sort'];
            $order = $request['order'];
            if ($order == 'ASC') {
                $new_order = 'DESC';
            } else {
                $new_order = 'ASC';
            }
        }
        
        if (isset($request['search'])) {
            $search = $request['search'];
        }
        
        $string = $this->buildNavigation($request, 'users.nickname', 'browseusers');
        
        if ($users != false) {
            $string .= $this->buildPagination($numberOfPages, $request, $users['count']);
            $string .= '
                <table width="75%" cellspacing="1" cellpadding="1" class="boxborder titlebarcolour" align="center">
                  <tr>
                    <td class="boxborder text">' . PHPST_CLICK_USER . '.</td>
                  </tr>
                </table>
                
                <table width="75%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                  <tr align="center" class="sectionscolour">
                    <td class="boxborder text">
                      <a href="' . $url . '?page=browseusers&amp;field=nickname&amp;search=' . $search . '&amp;sort=ID&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_USERID . '">
                        <b>' . PHPST_USERID . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?page=browseusers&amp;field=nickname&amp;search=' . $search . '&amp;sort=admin_status&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_ADMIN_STATUS . '">
                        <b>' . PHPST_ADMIN_STATUS . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?page=browseusers&amp;field=nickname&amp;search=' . $search . '&amp;sort=timestamp&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_DATE_JOINED . '">
                        <b>' . PHPST_DATE_JOINED . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?page=browseusers&amp;field=nickname&amp;search=' . $search . '&amp;sort=nickname&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_NAME . '">
                        <b>' . PHPST_NAME . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?page=browseusers&amp;field=nickname&amp;search=' . $search . '&amp;sort=username&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_INDEX_USERNAME . '">
                        <b>' . PHPST_INDEX_USERNAME . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?page=browseusers&amp;field=nickname&amp;search=' . $search . '&amp;sort=department_name&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_DEPARTMENT . '">
                        <b>' . PHPST_DEPARTMENT . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?page=browseusers&amp;field=nickname&amp;search=' . $search . '&amp;sort=email&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_EMAIL . '">
                        <b>' . PHPST_EMAIL . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?page=browseusers&amp;field=nickname&amp;search=' . $search . '&amp;sort=ticket_timestamp&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_TICKETS . '">
                        <b>' . PHPST_TICKETS_TITLE . '</b>
                      </a>
                    </td>
                  </tr>';
                
              foreach ($users as $key => $v) {
                  if ($key != 'count') {
                      $admin_status = $v['user']['admin_status'];
                      $date_created = gmdate('d-m-Y',$v['user']['timestamp']);
                      $nickname = $v['user']['nickname'];
                      $username = $v['user']['username'];
                      $ID = $v['user']['ID'];
                      $email = $v['user']['email'];
                      $department_ID = @$v['department']['ID'];
                      $department_name = @$v['department']['name'];
                      
                      $string .= "
                          <tr align=\"center\" bgcolor=\"#F4FAFF\">
                            <td class=\"boxborder list-menu\">
                              <a href=\"$url?page=viewuser&amp;ID=$ID\">
                                $ID
                              </a>
                            </td>
                            <td class=\"boxborder text\">$admin_status</td>
                            <td class=\"boxborder text\">$date_created</td>
                            <td class=\"boxborder text\">$nickname</td>
                            <td class=\"boxborder text\">$username</td>
                            <td class=\"boxborder text\">
                              <a href=\"$url?page=viewdepartment&amp;field=departments.ID&amp;search=$department_ID&amp;ID=$department_ID\">
                                $department_name
                              </a>
                            </td>
                            <td class=\"boxborder text\">
                              <a href=\"mailto:$email\">
                                $email
                              </a>
                            </td>
                            <td class=\"boxborder text\">";
                      $tickets_count = 0;
                      foreach ($v['tickets'] as $ticket) {
                          if ($ticket['status'] == 'Open') {
                              $tickets_count ++;
                          }
                      }
                      if (isset($v['tickets'])) {
                          $string .= "<a href=\"$url?page=browsetickets&amp;field=users.ID&amp;search=$ID\">
                                " . $tickets_count . "/" . @count($v['tickets']) ."
                              </a>";
                      } else {
                          $string .= "0";
                      }
                              
                      $string .= "</td>
                          </tr>";
                      
                  }
              }
              $string .= '</table>';
              $string .= $this->buildPagination($numberOfPages, $request, $users['count']);
        } else {
            $string .= '
                <div style="padding-top:5px"></div>
                <table width="75%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                  <tr>
                    <td class="text">' . PHPST_NO_RESULTS . '</td>
                  </tr>
                </table>';
        }
        return $string;
    }
    
    /**
    * Builds the browsedepartments page.
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildBrowsedepartments($request) {
        // Retrieve list of departments from DB
        $departments = FormHandler::get_departments($request);
        $url = $this->getAbsoluteUrl() . 'index.php';
        $search = 'all';
        
        $numberOfPages = ceil($departments['count'] / PHPST_MAX_RECORDS);
        $new_order = 'ASC';
        
        if (isset($request['sort']) && isset($request['order'])) {
            $sort = $request['sort'];
            $order = $request['order'];
            if ($order == 'ASC') {
                $new_order = 'DESC';
            } else {
                $new_order = 'ASC';
            }
        }
        
        if (isset($request['search'])) {
            $search = $request['search'];
        }
        
        $string = $this->buildNavigation($request, 'departments.nickname');
        
        if ($departments != false) {
            $string .= $this->buildPagination($numberOfPages, $request, $departments['count']);
            $string .= '
                <table width="75%" cellspacing="1" cellpadding="1" class="boxborder titlebarcolour" align="center">
                  <tr>
                    <td class="boxborder text">' . PHPST_CLICK_DEPARTMENT . '.</td>
                  </tr>
                </table>
                
                <table width="75%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                  <tr align="center" class="sectionscolour">
                    <td class="boxborder text">
                      <a href="' . $url . '?page=browsedepartments&amp;field=name&amp;search=' . $search . '&amp;sort=ID&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_USERID . '">
                        <b>' . PHPST_USERID . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                      <a href="' . $url . '?page=browsedepartments&amp;field=name&amp;search=' . $search . '&amp;sort=name&amp;order=' . $new_order . '" title="' . PHPST_ORDERBY . ' ' . PHPST_NAME . '">
                        <b>' . PHPST_NAME . '</b>
                      </a>
                    </td>
                    <td class="boxborder text">
                        <b>' . PHPST_TICKETS_TITLE . '</b>
                    </td>
                    <td class="boxborder text">
                        <b>' . PHPST_MODS . '</b>
                    </td>
                    <td class="boxborder text">
                        <b>' . PHPST_DELETE . '</b>
                    </td>
                  </tr>';
              
              foreach ($departments as $key => $v) {
                  if ($key != 'count') {
                      $name = $v['department']['name'];
                      $ID = $v['department']['ID'];
                      
                      $tickets_count = 0;
                      foreach ($v['tickets'] as $ticket) {
                          if ($ticket['status'] == 'Open') {
                              $tickets_count ++;
                          }
                      }
                      $users_count = @count($v['users']);
                      $string .= "
                          <tr align=\"center\" bgcolor=\"#F4FAFF\">
                            <td class=\"boxborder list-menu\">
                              <a href=\"$url?page=viewdepartment&amp;field=departments.ID&amp;search=$ID&amp;ID=$ID\">
                                $ID
                              </a>
                            </td>
                            <td class=\"boxborder text\">$name</td>
                            <td class=\"boxborder text\">
                              <a href=\"$url?page=browsetickets&amp;field=departments.ID&amp;search=$ID\">
                                $tickets_count/" . @count($v['tickets']) . "
                              </a>
                            </td>
                            <td class=\"boxborder text\">
                              <a href=\"$url?page=browseusers&amp;field=departments.ID&amp;search=$ID\">
                                $users_count
                              </a>
                            </td>
                            <td class=\"boxborder text\">
                              <button onclick=\"window.location='$url?page=deletedepartment&amp;ID=$ID';\">" . PHPST_DELETE . "</button>
                            </td>
                          </tr>";                  
                  }
              }
              
              $string .= '</table>';
              $string .= $this->buildPagination($numberOfPages, $request, $departments['count']);
        } else {
            $string .= '
                <div style="padding-top:5px"></div>
                <table width="75%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                  <tr>
                    <td class="text">' . PHPST_NO_RESULTS . '</td>
                  </tr>
                </table>';
        }
        return $string;
    }
    
    /**
    * Builds the deleteDepartment page.
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildDeletedepartment($request) {
        $url = $this->getAbsoluteUrl() . 'index.php';
        $result = FormHandler::delete_department($request);
        $string = $this->buildNavigation($request);
        $string .= '<table width="75%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
              <tr>
                <td class="boxborder" width="50%" valign="top" style="padding-top:5px">
                    ' . PHPST_DELETE_DEPARTMENT_SUCCESS . '
                </td>
              </tr>
            </table>';
        return $string;
    }
    
    /**
    * Builds the viewuser page.
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildViewuser($request) {
        $url = $this->getAbsoluteUrl() . 'index.php';
        $user = FormHandler::get_user($request);
        
        $status_indices = array('Client' => 0, 'Mod' => 1, 'Admin' => 2);
        
        $field = '';
        if (is_array($this->getMsg())) {
            $msg = $this->getMsg();
            $error = $msg[0]['field'];
            $this->setMsg($msg[0]['message']);
        } 
        
        // If there are no users for this user, just create an empty array
        if (!isset($user['tickets'])) {
            $user['tickets'] = array();
        }
        
        $dep_dropdown = 'N/A';
        
        // Only show departments drop-down if user is a mod
        if ($user['user']['admin_status'] == 'Mod') {
            $dep_dropdown = parent::getDepartmentsDropDown($request, @$user['department']['name']);
            $dep_dropdown .= "<a href=\"$url?page=viewdepartment&amp;field=departments.ID&amp;search={$user['department']['ID']}&amp;ID={$user['department']['ID']}\">
                        " . PHPST_VIEW . '</a>';
        }
       
        $string = $this->buildNavigation($request);
        $string .= '
            <form enctype="multipart/form-data" action="' . $url . '" method="post">
                <input type="hidden" name="formdata" value="edituser" />
                <input type="hidden" name="page" value="viewuser" />
                <input type="hidden" name="field" value="users.ID" />
                <input type="hidden" name="search" value="' . $user['user']['ID'] . '" />
                <input type="hidden" name="username" value="' . $user['user']['username'] . '" />
                <input type="hidden" name="ID" value="' . $user['user']['ID'] . '" />
                <input type="hidden" name="email" value="' . $user['user']['email'] . '" />
                <input type="hidden" name="password" value="' . $user['user']['password'] . '" />
                <input type="hidden" name="timestamp" value="' . $user['user']['timestamp'] . '" />
                <input type="hidden" name="last_login" value="' . $user['user']['last_login'] . '" />
            <table width="75%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
              <tr>
                <td class="boxborder" width="50%" valign="top" style="padding-top:5px">
                
                  <table width="97%" cellspacing="1" cellpadding="1" class="boxborder" align="center">
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_NAME . ':</b></td>
                      <td class="boxborder text">
                        <input type="text" name="nickname" value="' . $user['user']['nickname'] . '" /></td>
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_USERNAME . ':</b></td>
                      <td class="boxborder text">' . $user['user']['username'] . '</td>
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_PASSWORD . ':</b></td>
                      <td class="boxborder text">' . $user['user']['password'] . '</td>
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_DEPARTMENT . ':</b></td>
                      <td class="boxborder text">' . $dep_dropdown . '</td>
                    </tr>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_ADMIN_STATUS . ':</b></td>
                      <td class="boxborder text">
                        <select id="admin_status" name="admin_status">
                          <option>Client</option>
                          <option>Mod</option>
                          <option>Admin</option>
                        </select>
                    </tr>
                    <script language="Javascript" type="text/Javascript">
                    <!--
                    document.getElementById("admin_status").selectedIndex = ' . $status_indices[$user['user']['admin_status']] . ';
                    -->
                    </script>
                    <tr>
                      <td bgcolor="#EEEEEE" class="boxborder text"><b>' . PHPST_EMAIL . ':</b></td>
                      <td class="boxborder text"><a href="mailto:' . $user['user']['email'] . '">' . $user['user']['email'] . '</a></td>
                    </tr>
                    <tr>
                      <td align="right">
                        <input type="submit" value="' . PHPST_SUBMIT .'" />
                      </td>
                    </tr>
                  </table>
                  
                  <div style="padding-top:5px"></div>
                  
                  <br />
                </td>
                
                <td width="50%" valign="top" style="padding-top:5px">
                  <table width="98%" cellspacing="1" cellpadding="1" class="boxborder" align="center">';
            if (isset($user['tickets'])) {
                // Remove closed tickets
                foreach ($user['tickets'] as $ID => $ticket) {
                    if ($ticket['status'] != 'Open') {
                        unset($user['tickets'][$ID]);
                    }
                }
            }
            
            // If any tickets are left, display them
            if (count($user['tickets']) > 0) {
                  $string .= '
                      <tr bgcolor="#AABBDD">
                        <td colspan="6" class="boxborder text"><b>' . PHPST_TICKETS . '</b></td>
                      </tr>
                      <tr align="center" class="sectionscolour">
                        <td class="boxborder text"><b>' . PHPST_TICKETID . '</b></td>
                        <td class="boxborder text"><b>' . PHPST_REPLIES . '</b></td>
                        <td class="boxborder text"><b>' . PHPST_SUBJECT . '</b></td>
                        <td class="boxborder text"><b>' . PHPST_TIME . '</b></td>
                        <td class="boxborder text"><b>' . PHPST_URGENCY . '</b></td>
                        <td class="boxborder text"><b>' . PHPST_STATUS . '</b></td>
                      </tr>';
              
                foreach ($user['tickets'] as $ID => $ticket) {
                    $myTicket = FormHandler::get_ticket($ticket);
                    $answers = @count($myTicket['answers']);
                    $string .= '
                          <tr align="center" bgcolor="#F4FAFF">
                            <td class="boxborder list-menu"><a href="' . $url . '?page=viewticket&amp;ID=' . $ticket['ID'] . '">' . $ticket['ID'] . '</a></td>
                            <td class="boxborder text">[' . $answers . ']</td>
                            <td class="boxborder text">' . $ticket['subject'] . '</td>
                            <td class="boxborder text">' . gmdate('Y m d H:i:s',$ticket['timestamp']) . '</td>
                            <td class="boxborder text" bgcolor="#' . eval('return PHPST_URGENCY_COLOUR_' . $ticket['urgency'] . ';') . '">' . eval('return PHPST_URGENCY_LABEL_' . $ticket['urgency'] . ';') . '</td>
                            <td class="boxborder text">
                            <span style="color:#000000">' . $ticket['status'] . '</span></td>
                          </tr>';
                } // end foreach
            } else {
                $string .= '
                      <tr bgcolor="#AABBDD">
                        <td colspan="4" class="boxborder text"><b>' . PHPST_NO_TICKETS . '</b></td>
                      </tr>';
            }
            $string .= '
                    </table>
                  <div style="padding-top:5px"></div>
                </td>
              </tr>
            </table>
            </form>';
        return $string;
    }
} 
?>
