<?php
/**
 * File containing the ModGUI class.
 * 
 * @category Class
 * @package PHPSupportTickets
 * @author Ian Warner, <iwarner@triangle-solutions.com> 
 * @author Nicolas Connault, <nicou@sweetpeadesigns.com.au> 
 * @copyright (C) 2001 Triangle Solutions Ltd
 * @version CVS: $Id: class.ModGUI.php,v 1.11 2005/08/17 08:47:33 nicconn Exp $
 * @since File available since Release 1.1.1.1
 * \\||
 */

/**
* user defined includes.
* @include
*/
require_once('abstract.GUI.php');

/**
 * AdminGUI is the Admin's User Interface class.
 * 
 * @access public
 * @package PHPSupportTicket
 */
class ModGUI extends GUI{
    /// --- FIELDS ---
    
    // --- METHODS ---
    /**
    * Constructor method.
    * 
    * @access public
    * @param string $absoluteURL
    * @param array $request
    */
    public function __construct($absoluteURL, $request) {
        parent::__construct($absoluteURL, $request);
        $this->buildHeader();
        $this->buildBody($request);
        $this->buildFooter();
    }
    
    /**
    * Updates this GUI using the $_REQUEST data sent from the previous page.
    * 
    * @access public
    * @param array $request
    * @param array $files (optional)
    * @return void
    */
    public function update($request, $files = null) {
        parent::update($request, $files);
    }
    
    /**
    * Builds the body of the GUI based on its current state.
    * 
    * @access public
    * @return void
    * @param array $request
    */
    public function buildBody($request) {
        if ($this->getPage() == '') {
            $this->setPage("home");
        }
        
        // Dynamically prepare the build function matching the requested page.
        $method = "build" . ucfirst($this->getPage());
        if (in_array($method, get_class_methods($this))) {
            $this->addHTML($this->$method($request));
        } else {
            $this->addHTML($this->buildHome($request));
        }
    }
    
    /**
    * Builds the home page (browse tickets).
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildBrowsetickets($request) {
        // Retrieve list of tickets from DB
        $ID = $this->user->getInfo('department_ID'); 
        
        if (isset($request['ticket_type']) && $request['ticket_type'] == 'my') {
            $ID = $this->user->getID();
        } 
        $tickets = FormHandler::get_tickets('Mod', $request, $ID);
        return parent::buildBrowsetickets($request, $tickets);
    }
    
    /**
    * Builds the navigation HTML.
    *  
    * @access public
    * @param array $request
    * @param string $searchterm
    * @param string $page
    * @return string
    */
    public function buildNavigation($request, $searchterm = 'tickets.subject', $page = 'home') {
        $string = parent::buildSearchBox($searchterm, $page);
        $selection_title = parent::buildSelectionTitle($request);
        $url = $this->getAbsoluteUrl() . 'index.php';
        $string .= '
                <script language="JavaScript" type="text/javascript">
                <!--
                function showMenu(value) {
                    hideMenu("myticketspan");
                    hideMenu("depticketspan");
                    document.getElementById(value).style.display = "block";
                }

                function hideMenu(value) {
                    if (value == "all") {
                        hideMenu("myticketspan");
                        hideMenu("depticketspan");
                    } else {
                        document.getElementById(value).style.display = "none";
                    }
                }
                -->
                </script>
                <table width="75%" cellspacing="1" cellpadding="1" class="boxborder titlebarcolour" align="center">
                  <tr>
                    <td class="boxborder text"><a href="' . $url . '" title="' . PHPST_STTITLE . '">' . PHPST_TITLELINK .'</a> - ' . PHPST_STTITLE . '</td>
                    <td id="optionmenu" class="boxborder list-menu">
                        <a href="' . $url . '?page=viewdepartment&amp;field=departments.ID&amp;search=' . $this->user->getInfo('department_ID') . '&amp;ID=' . $this->user->getInfo('department_ID') . '" onmouseover="showMenu(\'all\');" title="' . PHPST_DEPARTMENT . '">' . PHPST_DEPARTMENT .'</a>
                    </td>
                    <td id="optionmenu" class="boxborder list-menu">
                        <a href="' . $url . '?page=newticket" onmouseover="showMenu(\'all\');" title="' . PHPST_TITLEREQ . '">' . PHPST_TITLEREQ .'</a>
                    </td>
                    <td id="myticketmenu" class="boxborder list-menu">
                      <a href="' . $url . '?page=browsetickets&amp;ticket_type=my&amp;field=status&amp;search=all" onmouseover="showMenu(\'myticketspan\');">' . PHPST_MYTICKETS . '</a>
                    </td>
                    <td id="depticketmenu" class="boxborder list-menu">
                      <a href="' . $url . '?page=browsetickets&amp;ticket_type=department&amp;field=status&amp;search=all" onmouseover="showMenu(\'depticketspan\');">' . PHPST_DEPTICKETS . '</a>
                    </td>
                    <td id="optionmenu" class="boxborder list-menu" onmouseover="showMenu(\'all\');">
                      <a href="' . $url . '?logout">' . PHPST_TITLELOG . '</a>
                    </td>
                  </tr>
                  <tr>
                    <td style="text-align: left; height: 25px;">' . $selection_title . '</td>
                    <td colspan="6" style="text-align: right; height: 25px;">
                      <span id="myticketspan" class="submenu">
                        ' . PHPST_MYTICKETS . ' ::  
                        <a href="' . $url . '?page=browsetickets&amp;ticket_type=my&amp;field=status&amp;search=all" title="' . PHPST_ALL . '">' . PHPST_ALL .'</a>
                        <a href="' . $url . '?page=browsetickets&amp;ticket_type=my&amp;field=status&amp;search=open" title="' . PHPST_TITLEOPE . '">' . PHPST_TITLEOPE .'</a>
                        <a href="' . $url . '?page=browsetickets&amp;ticket_type=my&amp;field=status&amp;search=closed" title="' . PHPST_TITLECLO . '">' . PHPST_TITLECLO .'</a>
                      </span>
                      <span id="depticketspan" class="submenu">
                        ' . PHPST_DEPTICKETS . ' :: 
                        <a href="' . $url . '?page=browsetickets&amp;ticket_type=department&amp;field=status&amp;search=all" title="' . PHPST_ALL . '">' . PHPST_ALL .'</a>
                        <a href="' . $url . '?page=browsetickets&amp;ticket_type=department&amp;field=status&amp;search=open" title="' . PHPST_TITLEOPE . '">' . PHPST_TITLEOPE .'</a>
                        <a href="' . $url . '?page=browsetickets&amp;ticket_type=department&amp;field=status&amp;search=closed" title="' . PHPST_TITLECLO . '">' . PHPST_TITLECLO .'</a>
                      </span>
                    </td>
                  </tr>
                </table>';
        return $string;
    }
    
    /**
    * Redirects to this GUI's home page.
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildHome($request) {
        return $this->buildBrowsetickets($request);
    }
    
    /**
    * Builds the viewticket page (view tickets).
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildViewticket($request) {
        // Retrieve ticket info from DB
        // Note this is not a Ticket object, but an array containing a Ticket, a Department
        // and a User objects associated with that Ticket.
        $ticket = FormHandler::get_ticket($request);
        return parent::buildViewticket($request, $ticket);
    }
    
    /**
    * Builds the createuser page.
    * 
    * @access public
    * @return string
    * @param array $request
    */
    public function buildCreateuser($request) {
        return $this->buildHome($request);
    }
} 
?>