<?php
/**
 * File containing the AnswerSQLBuilder class.
 * 
 * @category Class
 * @package PHPSupportTickets
 * @author Ian Warner, <iwarner@triangle-solutions.com> 
 * @author Nicolas Connault, <nicou@sweetpeadesigns.com.au> 
 * @copyright (C) 2001 Triangle Solutions Ltd
 * @version CVS: $Id: class.AnswerSQLBuilder.php,v 1.3 2005/08/22 06:18:36 nicconn Exp $
 * @since File available since Release 1.1.1.1
 * \\||
 */


/**
* answer defined includes 
*/
require_once('abstract.SQLBuilder.php');

/**
* answer defined constants 
* @ignore
*/

/**
* This is a class with static functions used for creating SQL queries
* for the answers table
* 
* @package PHPSupportTickets
*/
class AnswerSQLBuilder extends SQLBuilder{

    /**
    * Constructor.
    */
    public function __construct() {
        parent::__construct();
    }
    
    /**
    * Creates a query for Inserting a new Answer into the DB.
    * 
    * @access public
    * @param object $answer The Answer to insert
    * @param array $columns The columns of the table in which to insert the Answer
    * @return string $query
    */
    public function insert($answer, $columns) {
        $query = "INSERT INTO answers SET ";
        
        // We use the DB columns as a basis for this loop, to avoid DB errors
        foreach ($columns as $k => $v) {             
            if ($v != "ID") {
                $value = eval('return htmlentities($answer->get' . ucfirst($v) . "(), ENT_QUOTES);");                
                $query .= "$v = '$value',";
            } 
        }

        // Trim last comma
        $query = rtrim($query,",");
        return $query;
    }
    
    /**
    * Creates a query for deleting Answer from the DB
    * 
    * @access public
    * @param int $answer_ID The answer's ID
    * @return string $query
    */
    public function delete($answer_ID) {
        return "DELETE FROM answers WHERE ID = '$answer_ID'";
    }
    
    /**
    * Creates a query for updating a Answer record in the DB from
    * the given Answer PHP object's data.
    * 
    * @access public
    * @param object $answer
    * @param array $columns The table's columns
    * @return string $query
    */
    public function update($answer, $columns) {
        $query = "UPDATE answers SET ";
        
        // We use the DB columns as a basis for this loop, to avoid DB errors
        foreach ($columns as $k => $v) {             
            if ($v != "ID") {
                $value = eval('return $answer->get' . ucfirst($v) . "();");                
                $query .= "$v = '$value',";
            }
        }
        // Trim last comma
        $query = rtrim($query, ",");
        $query .= " WHERE ID = " . $answer->getID(). "";
        return $query;
    }
    
    /**
    * Creates a query for retrieving all data about this Answer from the DB,
    * including all data from other tables linked through foreign keys.
    * 
    * @access public
    * @param array An array of fields used to customise the query
    * @return string $query
    */
    public function get($fields) {
        $where = "";
        $myAnswer = null;
        $filter = " answers.ID >= 0 AND answers.ticket_ID = tickets.ID
                   AND answers.user_ID = users.ID AND tickets.department_ID = departments.ID ";
        $query = "SELECT answers.*
                    users.ID as 'user_ID',
                    users.nickname as 'user_nickname',
                    users.username as 'user_username',
                    users.password as 'user_password',
                    users.admin_status as 'user_admin_status',
                    tickets.user_ID as 'ticket_user_ID', 
                    tickets.ID as 'ticket_ID', 
                    tickets.body as 'ticket_body',
                    tickets.subject as 'ticket_subject',
                    tickets.timestamp as 'ticket_timestamp',
                    tickets.status as 'ticket_status',
                    tickets.urgency as 'ticket_urgency',
                    tickets.category as 'ticket_category',
                    tickets.department_ID as 'ticket_department_ID',
                    departments.ID as 'department_ID', 
                    departments.name as 'department_name', 
                    departments.description as 'department_description'
                    
                    FROM answers, tickets, users, departments
                    WHERE ";
        
        if (array_key_exists('timestamp', $fields)) {
            $timestamp = $fields['timestamp'];
            $filter .= " AND answers.timestamp = '$timestamp' ";
        } elseif (array_key_exists('ID', $fields)) {
            $ID = $fields['ID'];
            $filter .= " AND answers.ID = '$ID' ";
        } else {
            return false;
        }
        
        $query .= $filter;
        return $query;
    }
    
    /**
    * Creates a query for retrieving all data about a number of Answers from the DB,
    * including all data from other tables linked through foreign keys.
    * 
    * @param int $answer_ID
    * @param string $answer_type Admin or Mod
    * @param string $search The search term
    * @param string $field The field being searched
    * @param string $sort The field by which to sort the results
    * @param string $order The order in which to sort the results
    * @return string $query
    */
    public function getFullAnswersArray($answer_ID, $answer_type = "Mod", $search = "", $field = ""
            , $sort = "", $order = "") {
        $filter = " answers.ID >= 0 AND answers.ticket_ID = tickets.ID
                   AND answers.user_ID = users.ID AND tickets.department_ID = departments.ID ";
        if ($search != "") {
            // If the search is for "All" records, prepare query
            if ($search != "all" && $search != 'timestamp') {
                $filter = " AND $field LIKE '%$search%' ";
            } elseif ($search == 'timestamp') {
                $filter = " AND answers.timestamp LIKE '%$search%' ";
            }
        }
        $order = "ORDER BY $sort $order";
        
        $answers = array();
        
        // Perform a query that returns every answer and associated info
        $query = "
                SELECT answers.*,
                    users.ID as 'user_ID',
                    users.nickname as 'user_nickname',
                    users.username as 'user_username',
                    users.password as 'user_password',
                    users.admin_status as 'user_admin_status',
                    tickets.user_ID as 'ticket_user_ID', 
                    tickets.ID as 'ticket_ID', 
                    tickets.body as 'ticket_body',
                    tickets.subject as 'ticket_subject',
                    tickets.timestamp as 'ticket_timestamp',
                    tickets.status as 'ticket_status',
                    tickets.urgency as 'ticket_urgency',
                    tickets.category as 'ticket_category',
                    tickets.department_ID as 'ticket_department_ID',
                    departments.ID as 'department_ID', 
                    departments.name as 'department_name', 
                    departments.description as 'department_description'
                    
                    FROM answers, tickets, users, departments
                    WHERE ";
            $query .= " $filter $order ";
        return $query;
    }
}
?>