<?php
/**
* File holding the DepartmentSQLBuilder class
* @package contactmanagement
* @author Nicolas Connault, <nicou@sweetpeadesigns.com.au>
*/

/**
* department defined includes 
*/
require_once('abstract.SQLBuilder.php');

/**
* department defined constants 
* @ignore
*/

/**
* This is a class with static functions used for creating SQL queries
* for the departments table
*/
class DepartmentSQLBuilder extends SQLBuilder{

    /**
    * Constructor.
    */
    public function __construct() {
        parent::__construct();
    }
    
    /**
    * Creates a query for Inserting a new Department into the DB.
    * 
    * @access public
    * @param object $department The Department to insert
    * @param array $columns The columns of the table in which to insert the Department
    * @return string $query
    */
    public function insert($department, $columns) {
        $query = "INSERT INTO departments SET ";
        
        // We use the DB columns as a basis for this loop, to avoid DB errors
        foreach ($columns as $k => $v) {             
            if ($v != "ID") {
                $value = eval('return htmlentities($department->get' . ucfirst($v) . "(), ENT_QUOTES);");                
                $query .= "$v = '$value',";
            } 
        }

        // Trim last comma
        $query = rtrim($query,",");
        return $query;
    }

    /**
    * Creates a query for retrieving an array of Users 
    * attached to a Department.
    * 
    * @access public
    * @param int $department_ID The department's ID
    * @return string $query
    */
    public function getUsers($department_ID) {
        $query ="SELECT users.* FROM users, department_users WHERE 
            user_ID = users.ID AND department_ID = '$department_ID'";
        return $query;
    }
    
    /**
    * Creates a query for deleting Department from the DB
    * Some users may end up with no department linking to them.
    * 
    * @access public
    * @param int $department_ID The department's ID
    * @return string $query
    */
    public function delete($department_ID) {
        $query = "DELETE FROM departments WHERE departments.ID = '$department_ID' ";
        return $query;
    }
    
    /**
    * Creates a query for deleting user->department associations from the DB.
    * Some users may end up with no department linking to them.
    * 
    * @access public
    * @param int $department_ID The department's ID
    * @return string $query
    */
    public function deleteUsers($department_ID) {
        $query = "DELETE FROM department_users WHERE department_ID = '$department_ID' ";
        return $query;
    }
    
    /**
    * Creates a query for updating the list of Users.
    * 
    * @access public
    * @param int $department_ID The Department's ID
    * @param array $users The department's array of users
    * @return string $query
    */
    public function updateUsersList($department_ID, $users) {
        if (is_array($users) && count($users) > 0) {
            $query = "INSERT INTO department_users (department_ID, user_ID) VALUES ";
            foreach ($users as $k => $v) {
                $query .= "($department_ID, " . $v->getID() . "),";
            }
            // Erase last comma
            $query = rtrim($query, ",");
            return $query;
        } else {
            return null;
        }
    }
    
    /**
    * Creates a query for removing user entries in department_users table for this department
    * 
    * @access public
    * @param int $department_ID The Department's ID
    * @return string $query
    */
    public function emptyDepartmentUsers($department_ID) {
        $query = "DELETE FROM department_users WHERE department_ID = $department_ID; ";
        return $query;
    }
    
    /**
    * Creates a query for updating a Department record in the DB from
    * the given Department PHP object's data.
    * 
    * @access public
    * @param object $department
    * @param array $columns The table's columns
    * @return string $query
    */
    public function update($department, $columns) {
        $query = "UPDATE departments SET ";
        
        // We use the DB columns as a basis for this loop, to avoid DB errors
        foreach ($columns as $k => $v) {             
            if ($v != "ID") {
                $value = eval('return $department->get' . ucfirst($v) . "();");                
                $query .= "$v = '$value',";
            }
        }
        // Trim last comma
        $query = rtrim($query, ",");
        $query .= " WHERE ID = " . $department->getID(). "";
        return $query;
    }
    
    /**
    * Creates a query for retrieving a complete Department Object
    * 
    * @access public
    * @param array An array of fields used to customise the query
    * @return string $query
    */
    public function get($fields) {
        $filter = "departments.ID >= 0 ";
        $query = "SELECT * FROM departments WHERE ";
        
        if (is_array($fields) && array_key_exists('name', $fields)) {
            $name = $fields['name'];   
            $filter .= "AND name = '$name'";
        } elseif (array_key_exists('ID', $fields) || array_key_exists('departments.ID', $fields)) {
            $ID = $fields['ID'];
            $filter .= "AND ID = '$ID'";
        } elseif(isset($fields['field']) && $fields['field'] == 'departments.ID') {
            $ID = $fields['search'];
            $filter .= "AND ID = '$ID'";
        } else {
            return false;
        }
        $query .= " $filter ";
        
        return $query;
    }
    
    /**
    * Creates a query for retrieving all data about a number of Departments from the DB,
    * including all data from other tables linked through foreign keys.
    * 
    * @param int $department_ID
    * @param string $department_type Admin or Mod
    * @param string $search The search term
    * @param string $field The field being searched
    * @param string $sort The field by which to sort the results
    * @param string $order The order in which to sort the results
    * @return string $query
    */
    public function getFullDepartmentsArray($department_type = "Mod", 
            $search = null, $field = null, $sort = null, $order = null) {
            
        $filter = "";
        if (!is_null($search) && $search != '') {
            // If the search is for "All" records, prepare query
            if (eregi('ID', $field)) {
                $filter = "AND $field = $search ";
            } elseif ($search != "all") {
                $filter = "AND $field LIKE '%$search%' ";
            }
        }
        if (!is_null($order) && !is_null($sort) && $order != '') {
            $order = "ORDER BY $sort $order";
        } else {
            $order = "ORDER BY name ASC";
        }
        
        // Perform a query that returns every department, its mods and tickets
        $query = "
                /* Departments with mods AND tickets */
                SELECT DISTINCT
                departments.ID AS 'ID', 
                departments.name AS 'name',
                departments.description AS 'description',
                
                users.ID AS 'user_ID', 
                users.nickname AS 'user_nickname',
                users.username AS 'user_username', 
                users.password AS 'user_password', 
                users.email AS 'user_email', 
                users.timestamp AS 'user_timestamp', 
                users.last_login AS 'user_last_login', 
                users.admin_status AS 'user_admin_status',
                
                tickets.ID AS 'ticket_ID', 
                tickets.user_ID AS 'ticket_user_ID',
                tickets.subject AS 'ticket_subject', 
                tickets.body AS 'ticket_body',
                tickets.timestamp AS 'ticket_timestamp', 
                tickets.status AS 'ticket_status',
                tickets.urgency AS 'ticket_urgency', 
                tickets.department_ID AS 'ticket_department_ID'
                
                FROM departments, department_users, users, tickets
                WHERE department_users.department_ID = departments.ID
                AND tickets.department_ID = departments.ID
                AND department_users.user_ID = users.ID 
                $filter
                
                UNION 
                
                /* Departments with mods AND NO tickets */
                SELECT  DISTINCT
                departments.ID AS 'department_ID', 
                departments.name AS 'department_name', 
                departments.description AS 'department_description',
                
                users.ID AS 'user_ID', 
                users.nickname AS 'user_nickname',
                users.username AS 'user_username', 
                users.password AS 'user_password', 
                users.email AS 'user_email', 
                users.timestamp AS 'user_timestamp', 
                users.last_login AS 'user_last_login', 
                users.admin_status AS 'user_admin_status',
                
                null AS 'ticket_ID', 
                null AS 'ticket_user_ID',
                null AS 'ticket_subject', 
                null AS 'ticket_body',
                null AS 'ticket_timestamp', 
                null AS 'ticket_status',
                null AS 'ticket_urgency', 
                null AS 'ticket_department_ID'
                
                FROM departments, department_users, users
                WHERE department_users.department_ID = departments.ID
                AND department_users.user_ID = users.ID 
                $filter
                
                UNION 
                
                /* Departments with NO mods AND WITH tickets */
                SELECT DISTINCT
                
                departments.ID AS 'ID', 
                departments.name AS 'name',
                departments.description AS 'description',
                
                null AS 'user_ID', 
                null AS 'user_nickname',
                null AS 'user_username', 
                null AS 'user_password', 
                null AS 'user_email', 
                null AS 'user_timestamp', 
                null AS 'user_last_login', 
                null AS 'user_admin_status',
                
                tickets.ID AS 'ticket_ID', 
                tickets.user_ID AS 'ticket_user_ID',
                tickets.subject AS 'ticket_subject', 
                tickets.body AS 'ticket_body',
                tickets.timestamp AS 'ticket_timestamp', 
                tickets.status AS 'ticket_status',
                tickets.urgency AS 'ticket_urgency', 
                tickets.department_ID AS 'ticket_department_ID'
                
                FROM departments, tickets
                WHERE departments.ID NOT IN (SELECT department_ID FROM department_users)
                AND tickets.department_ID = departments.ID
                $filter 
                
                UNION
                
                /* Departments with NO Mods AND NO tickets */
                SELECT DISTINCT
                departments.ID AS 'ID', 
                departments.name AS 'name',
                departments.description AS 'description',
                
                null AS 'user_ID', 
                null AS 'user_nickname',
                null AS 'user_username', 
                null AS 'user_password', 
                null AS 'user_email', 
                null AS 'user_timestamp', 
                null AS 'user_last_login', 
                null AS 'user_admin_status',
                
                null AS 'ticket_ID', 
                null AS 'ticket_user_ID',
                null AS 'ticket_subject', 
                null AS 'ticket_body',
                null AS 'ticket_timestamp', 
                null AS 'ticket_status',
                null AS 'ticket_urgency', 
                null AS 'ticket_department_ID'
                
                FROM departments
                WHERE departments.ID NOT IN (SELECT department_ID FROM department_users)
                AND departments.ID NOT IN (SELECT department_ID FROM tickets)
                $filter $order
                ";
        // die($query);
        return $query;
    }
}
?>