<?php
/**
 * File containing the TicketSQLBuilder class.
 * 
 * @category Class
 * @package PHPSupportTickets
 * @author Ian Warner, <iwarner@triangle-solutions.com> 
 * @author Nicolas Connault, <nicou@sweetpeadesigns.com.au> 
 * @copyright (C) 2001 Triangle Solutions Ltd
 * @version CVS: $Id: class.TicketSQLBuilder.php,v 1.12 2005/08/22 06:21:12 nicconn Exp $
 * @since File available since Release 1.1.1.1
 * \\||
 */

/**
* ticket defined includes 
*/
require_once('abstract.SQLBuilder.php');

/**
* ticket defined constants 
* @ignore
*/

/**
* This is a class with static functions used for creating SQL queries
* for the tickets table
* 
* @package PHPSupportTickets
*/
class TicketSQLBuilder extends SQLBuilder{

    /**
    * Constructor.
    */
    public function __construct() {
        parent::__construct();
    }
    
    /**
    * Creates a query for Inserting a new Ticket into the DB.
    * 
    * @access public
    * @param object $ticket The Ticket to insert
    * @param array $columns The columns of the table in which to insert the Ticket
    * @return string $query
    */
    public function insert($ticket, $columns) {
        $query = "INSERT INTO tickets SET ";
        
        // We use the DB columns as a basis for this loop, to avoid DB errors
        foreach ($columns as $k => $v) {             
            if ($v != "ID") {
                $value = eval('return htmlentities($ticket->get' . ucfirst($v) . "(), ENT_QUOTES);");
                $query .= "$v = '$value',";
            } 
        }

        // Trim last comma
        $query = rtrim($query,",");
        return $query;
    }
    
    /**
    * Creates a query for deleting Ticket from the DB
    * 
    * @access public
    * @param int $ticket_ID The ticket's ID
    * @return string $query
    */
    public function delete($ticket_ID) {
        return "DELETE FROM tickets WHERE ID = '$ticket_ID'";
    }
    
    /**
    * Creates a query for updating a Ticket record in the DB from
    * the given Ticket PHP object's data.
    * 
    * @access public
    * @param object $ticket
    * @param array $columns The table's columns
    * @return string $query
    */
    public function update($ticket, $columns) {
        $query = "UPDATE tickets SET ";
        
        // We use the DB columns as a basis for this loop, to avoid DB errors
        foreach ($columns as $k => $v) {             
            if ($v != "ID") {
                $value = eval('return $ticket->get' . ucfirst($v) . "();");                
                $query .= "$v = '" . addslashes($value) . "',";
            }
        }
        // Trim last comma
        $query = rtrim($query, ",");
        $query .= " WHERE ID = " . $ticket->getID(). "";
        return $query;
    }
    
    /**
    * Creates a query for retrieving all data about this Ticket from the DB,
    * including all data from other tables linked through foreign keys.
    * 
    * @access public
    * @param array An array of fields used to customise the query
    * @return string $query
    */
    public function get($fields) {
        $where = "";
        $myTicket = null;
        $filter = " tickets.user_ID = users.ID AND tickets.ID >= 0 AND tickets.department_ID = departments.ID ";
        
        if (array_key_exists('timestamp', $fields)) {
            $timestamp = $fields['timestamp'];
            $filter .= " AND tickets.timestamp = '$timestamp' ";
        } elseif (array_key_exists('ID', $fields)) {
            $ID = $fields['ID'];
            $filter .= " AND tickets.ID = '$ID' ";
        } else {
            return false;
        }
        
        $query = "SELECT tickets.*,
                    answers.ID as 'answer_ID', 
                    answers.user_ID as 'answer_user_ID', 
                    answers.ticket_ID as 'answer_ticket_ID', 
                    answers.body as 'answer_body',
                    answers.subject as 'answer_subject',
                    answers.timestamp as 'answer_timestamp',
                    answers.rating as 'answer_rating',
                    users.ID as 'user_ID',
                    users.nickname as 'user_nickname',
                    users.username as 'user_username',
                    users.password as 'user_password',
                    users.admin_status as 'user_admin_status',
                    users.timestamp as 'user_timestamp',
                    users.last_login as 'user_last_login',
                    users.email as 'user_email',
                    departments.ID as 'department_ID', 
                    departments.name as 'department_name', 
                    departments.description as 'department_description'
                    
                    FROM tickets, users, departments, answers
                    WHERE $filter AND answers.ticket_ID = tickets.ID
                    UNION
                    SELECT 
                    tickets.*,
                    null as 'answer_ID', 
                    null as 'answer_user_ID', 
                    null as 'answer_ticket_ID', 
                    null as 'answer_body',
                    null as 'answer_subject',
                    null as 'answer_timestamp',
                    null as 'answer_rating',
                    users.ID as 'user_ID',
                    users.nickname as 'user_nickname',
                    users.username as 'user_username',
                    users.password as 'user_password',
                    users.admin_status as 'user_admin_status',
                    users.timestamp as 'user_timestamp',
                    users.last_login as 'user_last_login',
                    users.email as 'user_email',
                    departments.ID as 'department_ID', 
                    departments.name as 'department_name', 
                    departments.description as 'department_description'
                    
                    FROM tickets, users, departments
                    WHERE $filter AND tickets.ID NOT IN (SELECT ticket_ID FROM answers) ";

        return $query;
    }
    
    /**
    * Creates a query for counting all tickets, using search criteria
    * 
    * @access public
    * @param string $admin_type Admin or Mod
    * @param string $search The search term
    * @param string $field The field being searched
    * @param string $sort The field by which to sort the results
    * @param string $order The order in which to sort the results
    * @param int $department_ID
    * @return string $query
    */
    public function getCount($admin_type = "Mod", $search = "", $field = ""
            , $sort = "", $order, $ID) {
        $filter = " tickets.ID >= 0 
                   AND tickets.user_ID = users.ID 
                   AND tickets.department_ID = departments.ID ";
        
        if ($admin_type == 'Mod') {
            $filter .= " AND departments.ID = '$ID' ";
        } elseif ($admin_type == 'Client') {
            $filter .= " AND users.ID = '$ID' ";
        }
        
        if ($search != "") {
            // If the search is for "All" records, prepare query
            if ($search != "all") {
                $filter .= "AND $field LIKE '%$search%' ";
            }
        }
        if ($order != '') {
            $order = "ORDER BY '$sort' $order";
        }
        
        $query = "SELECT COUNT(*) FROM tickets, users, departments WHERE $filter $order";
        return $query;
    }
    
    /**
    * Creates a query for retrieving all data about a number of Tickets from the DB,
    * including all data from other tables linked through foreign keys.
    * 
    * @param string $admin_type Admin or Mod
    * @param string $search The search term
    * @param string $field The field being searched
    * @param string $sort The field by which to sort the results
    * @param string $order The order in which to sort the results
    * @param int $department_ID or $user_ID
    * @param string $ticket_type 'department' tickets or 'my' tickets
    * @return string $query
    */
    public function getFullTicketsArray($admin_type = "Mod", $search = "", $field = ""
            , $sort = "", $order, $ID, $ticket_type = 'department') {
        
        $filter = " tickets.ID >= 0 
                   AND tickets.user_ID = users.ID ";
        
        if (strlen($ticket_type) < 1) {
            $ticket_type = 'department';
        }
        
        // Adjust filter according to admin_status
        if ($admin_type == 'Mod' && $ticket_type != 'my') {
            $filter .= " AND departments.ID = '$ID' ";
        } elseif ($admin_type == 'Client' || $ticket_type == 'my') {
            $filter .= " AND users.ID = '$ID' ";
        }
        
        // Adjust filter according to search term and value
        if ($search != "") {
            // If the search is for "All" records, prepare query
            if ($search != "all") {
                if ($field == 'status') {
                    $filter .= "AND `status` = '" . ucfirst($search) . "'";
                } else {
                    $filter .= "AND $field LIKE '%$search%' ";
                }
            } 
        }
        
        if ($order != '') {
            $order = "ORDER BY '$sort' $order";
        }
        
        $answers = array();
        
        // Perform a query that returns every ticket and associated info
        $query = "
                /* Tickets with answers AND a Department */
                SELECT tickets.*,
                    answers.ID as 'answer_ID', 
                    answers.user_ID as 'answer_user_ID', 
                    answers.ticket_ID as 'answer_ticket_ID', 
                    answers.body as 'answer_body',
                    answers.subject as 'answer_subject',
                    answers.timestamp as 'answer_timestamp',
                    answers.rating as 'answer_rating',
                    
                    users.ID as 'user_ID',
                    users.nickname as 'user_nickname',
                    users.username as 'user_username',
                    users.password as 'user_password',
                    users.admin_status as 'user_admin_status',
                    users.timestamp as 'user_timestamp',
                    users.last_login as 'user_last_login',
                    users.email as 'user_email',
                    
                    departments.ID as 'department_ID', 
                    departments.name as 'department_name', 
                    departments.description as 'department_description'
                    
                    FROM tickets, users, departments, answers
                    WHERE $filter AND answers.ticket_ID = tickets.ID
                    AND tickets.department_ID = departments.ID 
                    
                    UNION
                    
                /* Tickets with NO answers AND a Department*/    
                SELECT 
                    tickets.*,
                    
                    null as 'answer_ID', 
                    null as 'answer_user_ID', 
                    null as 'answer_ticket_ID', 
                    null as 'answer_body',
                    null as 'answer_subject',
                    null as 'answer_timestamp',
                    null as 'answer_rating',
                    
                    users.ID as 'user_ID',
                    users.nickname as 'user_nickname',
                    users.username as 'user_username',
                    users.password as 'user_password',
                    users.admin_status as 'user_admin_status',
                    users.timestamp as 'user_timestamp',
                    users.last_login as 'user_last_login',
                    users.email as 'user_email',
                    
                    departments.ID as 'department_ID', 
                    departments.name as 'department_name', 
                    departments.description as 'department_description'
                    
                    FROM tickets, users, departments
                    WHERE $filter AND tickets.ID NOT IN (SELECT ticket_ID FROM answers)
                    AND tickets.department_ID = departments.ID ";
        if ($admin_type != 'Mod' && !eregi('departments', $field)) {
            $query .= "
                    UNION
                    /* Tickets with answers but NO Department*/
                    SELECT tickets.*,
                        answers.ID as 'answer_ID', 
                        answers.user_ID as 'answer_user_ID', 
                        answers.ticket_ID as 'answer_ticket_ID', 
                        answers.body as 'answer_body',
                        answers.subject as 'answer_subject',
                        answers.timestamp as 'answer_timestamp',
                        answers.rating as 'answer_rating',
                        
                        users.ID as 'user_ID',
                        users.nickname as 'user_nickname',
                        users.username as 'user_username',
                        users.password as 'user_password',
                        users.admin_status as 'user_admin_status',
                        users.timestamp as 'user_timestamp',
                        users.last_login as 'user_last_login',
                        users.email as 'user_email',
                        
                        null as 'department_ID', 
                        null as 'department_name', 
                        null as 'department_description'
                        
                        FROM tickets, users, answers
                        WHERE $filter AND answers.ticket_ID = tickets.ID
                        AND tickets.department_ID = ''";
            if (!eregi('departments', $field)) {
                $query .= "UNION
                    
                        /* Tickets with NO answers and NO Department */
                        SELECT tickets.*,
                            null as 'answer_ID', 
                            null as 'answer_user_ID', 
                            null as 'answer_ticket_ID', 
                            null as 'answer_body',
                            null as 'answer_subject',
                            null as 'answer_timestamp',
                            null as 'answer_rating',
                            
                            users.ID as 'user_ID',
                            users.nickname as 'user_nickname',
                            users.username as 'user_username',
                            users.password as 'user_password',
                            users.admin_status as 'user_admin_status',
                            users.timestamp as 'user_timestamp',
                            users.last_login as 'user_last_login',
                            users.email as 'user_email',
                            
                            null as 'department_ID', 
                            null as 'department_name', 
                            null as 'department_description'
                            
                            FROM tickets, users
                            WHERE $filter AND tickets.ID NOT IN (SELECT ticket_ID FROM answers)
                            AND department_ID = '' 
                            ";
             }
        }
        return $query . $order;
    }
}
?>