<?php
/**
* File holding the UserSQLBuilder class
* @package contactmanagement
* @author Nicolas Connault, <nicou@sweetpeadesigns.com.au>
*/

/**
* user defined includes 
*/
require_once('abstract.SQLBuilder.php');

/**
* user defined constants 
* @ignore
*/

/**
* This is a class with static functions used for creating SQL queries
* for the users table
*/
class UserSQLBuilder extends SQLBuilder{

    /**
    * Constructor.
    */
    public function __construct() {
        parent::__construct();
    }
    
    /**
    * Creates a query for Inserting a new User into the DB.
    * 
    * @access public
    * @param object $user The User to insert
    * @param array $columns The columns of the table in which to insert the User
    * @return string $query
    */
    public function insert($user, $columns) {
        $query = "INSERT INTO users SET ";
        
        // We use the DB columns as a basis for this loop, to avoid DB errors
        foreach ($columns as $k => $v) {             
            if ($v != "ID") {
                $value = eval('return htmlentities($user->get' . ucfirst($v) . "(), ENT_QUOTES);");
                $query .= "$v = '$value',";
            } 
        }
        // Trim last comma
        $query = rtrim($query,",");
        return $query;
    }
    
    /**
    * Creates a query for deleting User from the DB
    * 
    * @access public
    * @param int $user_ID The user's ID
    * @return string $query
    */
    public function delete($user_ID) {
        return "DELETE FROM users WHERE ID = '$user_ID'";
    }
    
    /**
    * Creates a query for updating a User record in the DB from
    * the given User PHP object's data.
    * 
    * @access public
    * @param object $user
    * @param array $columns The table's columns
    * @return string $query
    */
    public function update($user, $columns) {
        $query = "UPDATE users SET ";
        
        // We use the DB columns as a basis for this loop, to avoid DB errors
        foreach ($columns as $k => $v) {             
            if ($v != "ID") {
                $value = eval('return $user->get' . ucfirst($v) . "();");                
                $query .= "$v = '$value',";
            }
        }
        // Trim last comma
        $query = rtrim($query, ",");
        $query .= " WHERE ID = " . $user->getID(). "";
        return $query;
    }
    
    /**
    * Creates a query for updating the Department associated with this User.
    * 
    * @access public
    * @param int $user_ID The user's ID
    * @param int $department_ID The department's ID 
    * @return string $query
    */
    public function updateDepartment_users($user_ID, $department_ID) {
        $query = "UPDATE department_users SET department_ID = $department_ID
                WHERE user_ID = $user_ID";
        return $query;
    }
    
    /**
    * Creates a query for retrieving all data about this User from the DB,
    * including all data from other tables linked through foreign keys.
    * 
    * @access public
    * @param array An array of fields used to customise the query
    * @return string $query
    */
    public function get($fields) {
        $where = "";
        $myUser = null;
        $filter = "";
        
        if (array_key_exists('username', $fields) 
                && array_key_exists('password', $fields)) {
            $username = $fields['username'];
            $password = $fields['password'];                
            $filter .= "AND username = '$username' 
                AND password = '$password'";
        } elseif (array_key_exists('ID', $fields) || array_key_exists('users.ID', $fields)) {
            $ID = $fields['ID'];
            $filter .= "AND users.ID = '$ID'";
        } else {
            return false;
        }
        
        $query = "SELECT users.*, 
                    departments.ID as 'department_ID',
                    departments.name as 'department_name'
                    FROM users, departments, department_users
                    WHERE users.ID >= 0 AND department_users.user_ID = users.ID
                        AND department_users.department_ID = departments.ID $filter
                    UNION
                  SELECT  users.*, 
                    null as 'department_ID',
                    null as 'department_name'
                    FROM users, departments, department_users
                    WHERE users.ID >= 0 AND users.ID NOT IN (SELECT user_ID FROM department_users) 
                    $filter";
        return $query;
    }
    
    
    /**
    * Creates a query for retrieving all data about a number of Users from the DB,
    * including all data from other tables linked through foreign keys.
    * 
    * @param string $user_type Admin or Mod
    * @param string $search The search term
    * @param string $field The field being searched
    * @param string $sort The field by which to sort the results
    * @param string $order The order in which to sort the results
    * @return string $query
    */
    public function getFullUsersArray($user_type = "Mod", $search = "", $field = ""
            , $sort = "", $order = "") {
        $filter = "";
        if ($search != "") {
            // If the search is for "All" records, prepare query
            if ($search != "all") {
                // If the search fields contains "department_name", do special search
                if ($field == "department_name") {
                    $filter = " AND departments.name LIKE '%$search%'";
                } elseif($field == 'ID' || $field == 'users.ID' || $field == 'departments.ID') {
                    $filter = "AND $field = $search ";
                } else {
                    $filter = "AND $field LIKE '%$search%' ";
                }
            }
        }
        if ($order != "") {
            $order = "ORDER BY $sort $order";
        } 
        
        $users = array();
        
        // Perform a query that returns every user, their department and tickets
        $query = "
                /* Users with departments AND tickets */
                SELECT 
                departments.ID AS 'department_ID', 
                departments.name AS 'department_name',
                departments.description AS 'department_description',
                
                users.ID AS 'ID', 
                nickname,
                username, 
                `password`, 
                email, 
                users.timestamp as 'timestamp', 
                last_login, 
                users.admin_status AS 'admin_status',
                
                tickets.ID AS 'ticket_ID', 
                tickets.user_ID AS 'ticket_user_ID',
                tickets.subject AS 'ticket_subject', 
                tickets.body AS 'ticket_body',
                tickets.timestamp AS 'ticket_timestamp', 
                tickets.status AS 'ticket_status',
                tickets.urgency AS 'ticket_urgency', 
                tickets.department_ID AS 'ticket_department_ID'
                
                FROM departments, department_users, users, tickets
                WHERE department_users.department_ID = departments.ID
                AND department_users.user_ID = users.ID AND tickets.user_ID = users.ID
                $filter";
        if (!eregi('tickets', $field)) {
            $query .= "UNION 
                
                /* Users with departments AND NO tickets */
                SELECT 
                departments.ID AS 'department_ID', 
                departments.name AS 'department_name', 
                departments.description AS 'department_description',
                
                users.ID AS 'ID', 
                nickname,
                username, 
                `password`, 
                email, 
                users.timestamp as 'timestamp', 
                last_login, 
                users.admin_status AS 'admin_status',
                
                null AS 'ticket_ID', 
                null AS 'ticket_user_ID',
                null AS 'ticket_subject', 
                null AS 'ticket_body',
                null AS 'ticket_timestamp', 
                null AS 'ticket_status',
                null AS 'ticket_urgency', 
                null AS 'ticket_department_ID'
                
                FROM departments, department_users, users
                WHERE department_users.department_ID = departments.ID
                AND department_users.user_ID = users.ID 
                AND users.ID NOT IN (SELECT user_ID FROM tickets)
                $filter";
        }
        if (!eregi('departments', $field)) {
            $query .= "UNION 
                
                /* Users with NO department AND WITH tickets */
                SELECT
                
                null AS 'department_ID', 
                null AS 'department_name', 
                null AS 'department_description',
                
                users.ID AS 'ID', 
                nickname,
                username, 
                `password`, 
                email, 
                users.timestamp as 'timestamp', 
                last_login, 
                users.admin_status AS 'admin_status',
                
                tickets.ID AS 'ticket_ID', 
                tickets.user_ID AS 'ticket_user_ID',
                tickets.subject AS 'ticket_subject', 
                tickets.body AS 'ticket_body',
                tickets.timestamp AS 'ticket_timestamp', 
                tickets.status AS 'ticket_status',
                tickets.urgency AS 'ticket_urgency', 
                tickets.department_ID AS 'ticket_department_ID'
                
                FROM users, tickets
                WHERE users.ID NOT IN (SELECT user_ID FROM department_users)
                AND tickets.user_ID = users.ID
                $filter";
        }
        if (!eregi('departments', $field) && !eregi('tickets', $field)) {
            $query .= "
                UNION
                
                /* Users with NO department AND NO tickets */
                SELECT 
                null AS 'department_ID', 
                null AS 'department_name', 
                null AS 'department_description',
                
                users.ID AS 'ID', 
                nickname,
                username, 
                `password`, 
                email, 
                users.timestamp as 'timestamp', 
                last_login, 
                users.admin_status AS 'admin_status',
                
                null AS 'ticket_ID', 
                null AS 'ticket_user_ID',
                null AS 'ticket_subject', 
                null AS 'ticket_body',
                null AS 'ticket_timestamp', 
                null AS 'ticket_status',
                null AS 'ticket_urgency', 
                null AS 'ticket_department_ID'
                
                FROM users
                WHERE users.ID NOT IN (SELECT user_ID FROM department_users)
                AND users.ID NOT IN (SELECT user_ID FROM tickets)
                $filter ";
        }
        // die($query);
        return $query . $order;
    }
}
?>