<?php
/**
 * File containing the FormHandler class.
 * 
 * @category Class
 * @package PHPSupportTickets
 * @author Ian Warner, <iwarner@triangle-solutions.com> 
 * @author Nicolas Connault, <nicou@sweetpeadesigns.com.au> 
 * @copyright (C) 2001 Triangle Solutions Ltd
 * @version CVS: $Id: static.FormHandler.php,v 1.19 2005/09/01 02:40:55 nicconn Exp $
 * @since File available since Release 1.1.1.1
 * \\||
 */

/**
* user defined includes.
* @include
*/
require_once('classes/class.User.php');
require_once('classes/class.Answer.php');
require_once('classes/class.Department.php');
require_once('classes/class.Ticket.php');
require_once('classes/class.Note.php');

/**
 * This static function class is a relay between the GUI and the DatabaseMirror classes.
 * It processes the form data and returns error messages if such errors occur.
 * 
 * @access public
 * @package PHPSupportTicket
 */
class FormHandler{
    /// --- FIELDS ---
    
    /**
    * Array of authorised file types.
    * 
    * @static
    * @access public
    * @var array $allowed_types
    */
    public static $allowedtypes = array('image/pjpeg',
            'image/jpeg',
            'image/gif',
            'application/msword',
            'application/pdf',
            'text/plain',
            'application/zip');
    
    
    /**
    * Array of file types and their extensions.
    * 
    * @static
    * @access public
    * @var array $filetypes
    */
    public static $filetypes = 	array (	'image/pjpeg' => '.jpg',
            'image/jpeg' => '.jpg',
            'image/gif' => '.gif',
            'application/msword' => '.doc',
            'application/pdf' => '.pdf',
            'text/plain' => '.txt',
            'application/zip' => '.zip');

	
    // --- METHODS ---
    /**
    * Sends the request data to the proper method for processing.
    * 
    * @static
    * @access public
    * @param string $page
    * @param string $action
    * @param array $request
    * @param array $files (optional)
    * @return void
    */
    public static function submitForm($page, $action, $request, $files = null) {
        return eval ('return FormHandler::' . $action . '_' . $page . '($request, $files);');
    }
        
    /**
    * Processes form data for registering a User.
    * 
    * @static
    * @access public
    * @param array $request
    * @return array Array with next page or array of errors with their fields
    */
    public static function register_createuser($request) {
        $msg = '';
        $error = array();
        $result = User::validate($request);
        if ($result !== true) {
            return $result;
        } else {
            // Insert new User
            $inserted_user = new User($request);
            
            $result = $inserted_user->addToDB();
            
            // If option is on, send email to all admins and to new user
            if (OPTION_EMAIL_USER_WHEN_REGISTERED == 1) {
                $users = User::getArrayFromDB('Admin', 'Admin',
                        'admin_status', null, null, PHPST_MAX_RECORDS, 'all');
                $recipients = array();
                $recipients[] = $inserted_user->getEmail();
                
                if (is_array($users) && count($users) > 0) {
                    foreach ($users as $user) {
                        $recipients[] = $user['user']['email'];
                    }
                }
                $result = MailMan::newUserNotify($recipients);
                if ($result !== true) {
                    // Returns array of errors with error fields and messages
                    return array(0 => array("field" => "Email",
                        "message" => $result));
                }
            }
            
            // Insert user ID in department_users table
            // A department_ID of 0 means the user does not get added to a department
            if ($request['department_ID'] > 0) {
                $department = Department::getFromDB(array("ID" => $request['department_ID']));
                $department->addUser($inserted_user);
                $depresult = $department->updateDB();
            } else {
                $depresult = true;
            }
            if ($result === false || $depresult !== true) {
                $msg = PHPST_USER_INSERT_FAILURE;
            } elseif(is_null($result)) {
                $msg = PHPST_USER_ALREADY_EXISTS;
            } else {
                $msg = true;
            }
        }
        return $msg;
    }
    
    /**
    * Processes form data for editing a User's details.
    * 
    * @static
    * @access public
    * @param array $request
    * @return string Success or Failure message
    */
    public static function edituser_viewuser($request) {
        // Perform validation on user entry
        $result = User::validate($request);
        if ($result === true) {
            $user = new User($request);
            $result = $user->updateDB();
            if ($result) {
                // If department_ID is set, update department_users table
                if (isset($request['department_ID'])) {
                    $department = Department::getFromDB(array('ID' => $request['department_ID']));
                    if ($request['admin_status'] == 'Mod') {
                        $department->addUser($user);
                    } else {
                        $department->removeUser($request['ID']);
                    }
                    $department->updateDB();
                }
                return true;
            } else {
                return false;
            }
        } else {
            return $result;
        }
    }
    
    /**
    * Processes form data for editing a department's details.
    * 
    * @static
    * @access public
    * @param array $request
    * @return string Success or Failure message
    */
    public static function editdepartment_viewdepartment($request) {
        // Perform validation on user entry
        $result = Department::validate($request);
        if ($result === true) {
            $department = Department::getFromDB($request);
            $department->setName($request['name']);
            $department->setDescription($request['description']);
            $result = $department->updateDB();
            if ($result) {
                return true;
            } else {
                return false;
            }
        } else {
            return $result;
        }
    }
    
    /**
    * Processes form data for verifying a User's login.
    * 
    * @static
    * @access public
    * @param array $request
    * @return mixed Success or Failure message
    */
    public static function verify_login($request) {
        $user = User::getFromDB($request);
        // print_r($request);
        if (is_object($user)) {
            return $user->getAdmin_status();
        } else {
            return false;
        }
    }
    
    /**
    * Processes form data for adding an Answer to a Ticket
    * 
    * @static
    * @access public
    * @param array $request
    * @param array $files (optional)
    * @return mixed Success or Failure message
    */
    public static function addanswer_viewticket($request, $files = null) {
        $result = Answer::validate($request);
        if ($result === true) {
            $answer = new Answer($request);
            $user = User::getFromDB(array('ID' => $request['user_ID']));
            
            if ($answer->addToDB()) {
                // If file is being uploaded, send it to handling function
                if (isset($files) && is_array($files)) {
                    $result = FormHandler::handleFileUpload($files, $user, $answer);
                    if ($result !== true) {
                        // Returns array of errors with error fields and messages
                        return array(0 => array("field" => "File Upload",
                            "message" => $result));
                    }
                }
                
                if (OPTION_EMAIL_CLIENT_WHEN_NEW_ANSWER == 1) {
                    $recipients = $user->getEmail();
                    $result = MailMan::newAnswerNotify($recipients, $answer);
                    if ($result !== true) {
                        return $result;
                    }
                }
                return true;
            } else {
                return false;
            }
        } else {
            return $result;
        }
    }
    
    /**
    * Retrieves a list (array) of tickets from the DB.
    * 
    * @static
    * @access public
    * @param string $admin_type Admin or Mod
    * @param string $search
    * @return array Tickets
    */
    public static function get_tickets($admin_type, $request, $department_ID) {
        $tickets = Ticket::getArrayFromDB($admin_type, @$request['search'], 
                @$request['field'], @$request['sort'], @$request['order'], PHPST_MAX_RECORDS, 
                @$request['rs_page'], $department_ID, @$request['ticket_type']);
        return $tickets;
    }
    
    /**
    * Retrieves a single ticket Object from the DB.
    * 
    * @static
    * @access public
    * @param string $admin_type Admin or Mod
    * @param string $search
    * @return array Tickets
    */
    public static function get_ticket($request) {
        $ticket = Ticket::getFromDB($request);
        
        // Update ticket status if requested
        if (isset($request['action'])) {
            if ($request['action'] == 'closeticket') {
                $ticket->setStatus('Closed');
            } elseif ($request['action'] == 'openticket') {
                $ticket->setStatus('Open');
            }
            $ticket->updateDB();
        }
        $ID = null;
        if (isset($request['ID'])) {
            $ID = $request['ID'];
        } else {
            $ID = $request['ticket_ID'];
        }
        $tickets = Ticket::getArrayFromDB('Admin', $ID, 'tickets.ID', null, null, 10, 1, null, null);
        
        $ticket = next($tickets);
        return $ticket;
    }
    
    /**
    * Retrieves an array of options from the DB.
    * 
    * @static
    * @access public
    * @return array Tickets
    */
    public static function get_options() {
        $options = array();
        $conn = &ADONewConnection(DSN);
        $query = "SELECT * FROM options";
        $rs = &$conn->Execute($query);
        if ($rs != false && $rs->RecordCount() > 0) {
            while (!$rs->EOF) {
                $options[] = $rs->fields;
                $rs->MoveNext();
            }
        }
        return $options;
    }
    
    /**
    * Updates options in the DB.
    * 
    * @static
    * @access public
    * @param int $option_ID
    * @param mixed $value
    * @return boolean result
    */
    public static function edit_options($request) {
        $conn = &ADONewConnection(DSN);
        foreach($request as $var => $value) {
            if (substr($var, 0, 6) == 'OPTION') {
                $query = "UPDATE options SET value = '$value' WHERE name = '$var'";
                $rs = &$conn->Execute($query);
            }
        }
    }
    
    /**
    * Retrieves a single user Object from the DB.
    * 
    * @static
    * @access public
    * @param array $request
    * @return array User
    */
    public static function get_user($request) {
        $user = User::getFromDB($request);
        
        // Update user status if requested
        if (isset($request['action'])) {
            if ($request['action'] == 'closeuser') {
                $user->setStatus('Closed');
            } elseif ($request['action'] == 'openuser') {
                $user->setStatus('Open');
            }
            $user->updateDB();
        }
        
        $ID = null;
        if (isset($request['ID'])) {
            $ID = $request['ID'];
        } else {
            $ID = $request['user_ID'];
        }
        $users = User::getArrayFromDB('Admin', $ID, 'users.ID', null, null, 10, 1, null);
        // die($users['count']);
        $user = next($users);
        return $user;
    }
    
    /**
    * Retrieves a list of users from the DB.
    * 
    * @static
    * @access public
    * @param array $request
    * @return array Users
    */
    public static function get_users($request) {
        $users = User::getArrayFromDB('Admin', @$request['search'], 
                @$request['field'], @$request['sort'], @$request['order'], PHPST_MAX_RECORDS, 
                @$request['rs_page']);
        return $users;
    }
    
    /**
    * Retrieves a single department Object from the DB.
    * 
    * @static
    * @access public
    * @param array $request
    * @return array Department
    */
    public static function get_department($request) {
        $department = Department::getFromDB($request);
        
        $ID = null;
        if (isset($request['ID'])) {
            $ID = $request['ID'];
        } else {
            $ID = @$request['department_ID'];
        }
        $departments = Department::getArrayFromDB('Admin', $ID, 'departments.ID', null, null, 10, 1, null);
        // die($departments['count']);
        $department = next($departments);
        return $department;
    }
    
    /**
    * Deletes a Department from the DB
    * 
    * @static
    * @access public
    * @param array $request
    * @return boolean True if successful
    */
    public static function delete_department($request) {
        $department = Department::getFromDB($request);
        return $department->removeFromDB($department->getID());
    }
    
    /**
    * Retrieves a list of departments from the DB.
    * 
    * @static
    * @access public
    * @param array $request
    * @return array Departments
    */
    public static function get_departments($request) {
        $departments = Department::getArrayFromDB('Admin', @$request['search'], 
                @$request['field'], @$request['sort'], @$request['order'], PHPST_MAX_RECORDS, 
                @$request['rs_page']);
        return $departments;
    }
    
    /**
    * Adds a new Ticket to the DB.
    * 
    * @static
    * @access public
    * @param array $request
    * @param array $files Optional upload file
    * @return boolean
    */
    public static function newticket_newticket($request, $files = null) {
        $result = Ticket::validate($request);
        if ($result === true) {
            $ticket = new Ticket($request);
            $user = User::getFromDB(array('ID' => $request['user_ID']));
            
            // die(print_r($ticket));
            if ($ticket->addToDB()) {
                // If file is being uploaded, send it to handling function
                if (isset($files) && is_array($files)) {
                    $result = FormHandler::handleFileUpload($files, $user, $ticket);
                    if ($result !== true && $result != PHPST_UPLOAD_NO_ATTACHMENT) {
                        // Returns array of errors with error fields and messages
                        return array(0 => array("field" => "File Upload",
                            "message" => $result));
                    }
                }
                
                // If option is on, send email to all mods in that department
                if (OPTION_EMAIL_MODS_WHEN_NEW_TICKET == 1) {
                    
                    $users = User::getArrayFromDB('Admin', $request['department_ID'],
                            'departments.ID', null, null, PHPST_MAX_RECORDS, 'all');
                    if (is_array($users) && count($users) > 0) {
                        $recipients = array();
                        foreach ($users as $user) {
                            $recipients[] = $user['user']['email'];
                        }
                        $result = MailMan::newTicketNotify($recipients, $ticket);
                        if ($result !== true) {
                            // Returns array of errors with error fields and messages
                            return array(0 => array("field" => "Email",
                                "message" => $result));
                        }
                    }
                }
                return true;
            } else {
                return false;
            }
        } else {
            return $result;
        }
    }
    
    /**
    * Adds a new Ticket to the DB.
    * 
    * @static
    * @access public
    * @param array $request
    * @return boolean
    */
    public static function newdepartment_createdepartment($request) {
        $result = Department::validate($request);
        if ($result === true) {
            $department = new Department($request);
            if ($department->addToDB()) {
                return true;
            } else {
                return array(0 => array('field' => 'name', 'message' => PHPST_DEPARTMENT_EXISTS));
            }
        } else {
            return $result;
        }
    }
    
    /**
    * Change tickets' status in the array to the value chosen (Open or closed).
    * 
    * @static
    * @access public
    * @param array $request
    * @return boolean
    */
    public static function changetickets_browsetickets($request) {
        if (isset($request['ticket'])) {
            foreach($request['ticket'] as $ticket_ID) {
                $ticket = Ticket::getFromDB(array("ID" => $ticket_ID));
                
                $ticket->setStatus($request['status']);
                $ticket->updateDB();
            }
        }
    }
    
    public static function verify_home() {}
    
    /**
    * When a file is uploaded with a ticket or answer, this
    * function is used to verify the file, create the appropriate
    * folder structure and save the file.
    * 
    * @static
    * @access public
    * @param array $files uploaded files
    * @param object $user
    * @param object $object
    */
    public static function handleFileUpload($files, $user, $object) {
        $type = strtolower(get_class($object));
        
        if ($files['userfile']['error'] == '4') {
            $msg = PHPST_UPLOAD_NO_ATTACHMENT;
        } elseif ($files['userfile']['error'] == '2') {
            $msg = PHPST_UPLOAD_TOO_BIG_FOR_TOOL;
        } elseif ($files['userfile']['error'] == '1') {
            $msg = PHPST_UPLOAD_TOO_BIG_FOR_PHP;
        } elseif ($files['userfile']['error'] == '3') {
            $msg = PHPST_UPLOAD_PARTIAL;
        } 
        
        // CHECK TO MAKE SURE THE UPLOADED FILE IS OF A FILE WE ALLOW AND GET THE NEWFILE EXTENSION
        elseif (!in_array($files['userfile']['type'], FormHandler::$allowedtypes)) {
            $msg = PHPST_UPLOAD_FILETYPE . $files['userfile']['type'] . PHPST_UPLOAD_NOT_ALLOWED;
    
            while ($type = current(FormHandler::$allowedtypes)) {
                $msg .= '<br />' . FormHandler::$filetypes[$type] . ' (' . $type . ')';
                next(FormHandler::$allowedtypes);
            } 
        } 
        
        // IF FILE IS NOT OVER SIZE AND IS CORRECT TYPE THEN CONTINUE WITH PROCESS
        elseif ($files['userfile']['error'] == '0') {
            $newfilename = strtolower($files['userfile']['name']); 
            // PRINT OUT THE RESULTS
            $msg = '<p><b>' . PHPST_UPLOAD_ATTACHMENT_SUCCESS . '</b> - ' . PHPST_UPLOAD_SUBMITTED . $files['userfile']['name'] . '
    				' . PHPST_UPLOAD_SIZE .  $files['userfile']['size'] . ' bytes -
    				' . PHPST_UPLOAD_TYPE . $files['userfile']['type'];
            
            // Check whether this user already has a folder in his/her name
            $user_folder = PHPST_UPLOAD_PATH . $user->getUsername();
            if (!is_dir($user_folder)) {
                // Create the directory
                mkdir($user_folder, 0700);
            }
            
            // Create the folder for this file (e.g. /uploads/username/ticket_43)
            $object_folder = $user_folder . '/' . $type . '_' . $object->getID();
            mkdir($object_folder, 0700);
            $result = move_uploaded_file($files['userfile']['tmp_name'], $object_folder . '/' . $newfilename);
            
            if ($result === true) {
                return $result;
            } else {
                return $result;
            }
        }
        return $msg;
    }
} 
?>
